@@ -343,8 +343,7 @@ void SubarrayPartitionerSparseFx::test_subarray_partitioner(
   Subarray subarray;
   create_subarray(array_->array_, ranges, subarray_layout, &subarray);
 
-  ThreadPool tp;
-  CHECK(tp.init(4).ok());
+  ThreadPool tp(4);
   Config config;
   SubarrayPartitioner subarray_partitioner(
       &config,
@@ -374,8 +373,7 @@ void SubarrayPartitionerSparseFx::test_subarray_partitioner(
   Subarray subarray;
   create_subarray(array_->array_, ranges, subarray_layout, &subarray);
 
-  ThreadPool tp;
-  CHECK(tp.init(4).ok());
+  ThreadPool tp(4);
   Config config;
   SubarrayPartitioner subarray_partitioner(
       &config,
@@ -403,8 +401,7 @@ void SubarrayPartitionerSparseFx::test_subarray_partitioner(
   Subarray subarray;
   create_subarray(array_->array_, ranges, subarray_layout, &subarray);
 
-  ThreadPool tp;
-  CHECK(tp.init(4).ok());
+  ThreadPool tp(4);
   Config config;
   SubarrayPartitioner subarray_partitioner(
       &config,
@@ -685,8 +682,7 @@ TEST_CASE_METHOD(
 
   create_subarray(array_->array_, ranges, subarray_layout, &subarray);
 
-  ThreadPool tp;
-  CHECK(tp.init(4).ok());
+  ThreadPool tp(4);
   Config config;
   SubarrayPartitioner subarray_partitioner(
       &config,
@@ -2286,12 +2282,11 @@ TEST_CASE_METHOD(
 
   // Check unsplittable
   tiledb::sm::Subarray subarray(
-      array->array_, layout, &g_helper_stats, g_helper_logger());
+      array->array_.get(), layout, &g_helper_stats, g_helper_logger());
   Range r;
   r.set_str_range("bb", "bb");
   subarray.add_range(0, std::move(r), true);
-  ThreadPool tp;
-  CHECK(tp.init(4).ok());
+  ThreadPool tp(4);
   Config config;
   SubarrayPartitioner partitioner(
       &config,
@@ -2325,7 +2320,7 @@ TEST_CASE_METHOD(
 
   // Check full
   tiledb::sm::Subarray subarray_full(
-      array->array_, layout, &g_helper_stats, g_helper_logger());
+      array->array_.get(), layout, &g_helper_stats, g_helper_logger());
   r.set_str_range("a", "bb");
   subarray_full.add_range(0, std::move(r), true);
   SubarrayPartitioner partitioner_full(
@@ -2353,7 +2348,7 @@ TEST_CASE_METHOD(
 
   // Check split
   tiledb::sm::Subarray subarray_split(
-      array->array_, layout, &g_helper_stats, g_helper_logger());
+      array->array_.get(), layout, &g_helper_stats, g_helper_logger());
   r.set_str_range("a", "bb");
   subarray_split.add_range(0, std::move(r), true);
   SubarrayPartitioner partitioner_split(
@@ -2391,7 +2386,7 @@ TEST_CASE_METHOD(
 
   // Check no split 2 MBRs
   tiledb::sm::Subarray subarray_no_split(
-      array->array_, layout, &g_helper_stats, g_helper_logger());
+      array->array_.get(), layout, &g_helper_stats, g_helper_logger());
   r.set_str_range("bb", "cc");
   subarray_no_split.add_range(0, std::move(r), true);
   SubarrayPartitioner partitioner_no_split(
@@ -2421,7 +2416,7 @@ TEST_CASE_METHOD(
 
   // Check split 2 MBRs
   tiledb::sm::Subarray subarray_split_2(
-      array->array_, layout, &g_helper_stats, g_helper_logger());
+      array->array_.get(), layout, &g_helper_stats, g_helper_logger());
   r.set_str_range("bb", "cc");
   subarray_split_2.add_range(0, std::move(r), true);
   SubarrayPartitioner partitioner_split_2(
@@ -2560,12 +2555,11 @@ TEST_CASE_METHOD(
   }
 
   tiledb::sm::Subarray subarray(
-      array->array_, layout, &g_helper_stats, g_helper_logger());
+      array->array_.get(), layout, &g_helper_stats, g_helper_logger());
   Range r;
   r.set_str_range("cc", "ccd");
   subarray.add_range(0, std::move(r), true);
-  ThreadPool tp;
-  CHECK(tp.init(4).ok());
+  ThreadPool tp(4);
   Config config;
   SubarrayPartitioner partitioner(
       &config,
