@@ -45,10 +45,13 @@
 #else
 #include "tiledb/sm/filesystem/posix.h"
 #endif
+#include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/c_api/tiledb.h"
+#include "tiledb/sm/c_api/tiledb_serialization.h"
 #include "tiledb/sm/enums/encryption_type.h"
+#include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
-#include "tiledb/sm/misc/utils.h"
+#include "tiledb/storage_format/uri/parse_uri.h"
 
 #include <chrono>
 #include <climits>
@@ -81,6 +84,8 @@ struct ArrayFx {
   void create_dense_array(const std::string& path);
   static std::string random_name(const std::string& prefix);
   static int get_fragment_timestamps(const char* path, void* data);
+  void array_serialize_wrapper(
+      tiledb_array_t* array, tiledb_array_t* new_array);
 };
 
 static const std::string test_ca_path =
@@ -125,7 +130,7 @@ int ArrayFx::get_fragment_timestamps(const char* path, void* data) {
   auto data_vec = (std::vector<uint64_t>*)data;
   std::pair<uint64_t, uint64_t> timestamp_range;
   if (tiledb::sm::utils::parse::ends_with(
-          path, tiledb::sm::constants::ok_file_suffix)) {
+          path, tiledb::sm::constants::write_file_suffix)) {
     auto uri = tiledb::sm::URI(path);
     if (tiledb::sm::utils::parse::get_timestamp_range(uri, &timestamp_range)
             .ok())
@@ -353,6 +358,31 @@ void ArrayFx::create_dense_array(const std::string& path) {
   tiledb_array_schema_free(&array_schema);
 }
 
+void ArrayFx::array_serialize_wrapper(
+    tiledb_array_t* array, tiledb_array_t* new_array) {
+  // Serialize the array
+  tiledb_buffer_t* buff;
+  REQUIRE(
+      tiledb_serialize_array(
+          ctx_,
+          array,
+          (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
+          1,
+          &buff) == TILEDB_OK);
+
+  // Load array from the rest server
+  REQUIRE(
+      tiledb_deserialize_array(
+          ctx_,
+          buff,
+          (tiledb_serialization_type_t)tiledb::sm::SerializationType::CAPNP,
+          0,
+          &new_array) == TILEDB_OK);
+
+  // Clean up.
+  tiledb_buffer_free(&buff);
+}
+
 TEST_CASE_METHOD(
     ArrayFx, "C API: Test getting array URI", "[capi][array][array-uri]") {
   SupportedFsLocal local_fs;
@@ -635,7 +665,8 @@ TEST_CASE_METHOD(
 
     // Close arrays
     rc = tiledb_array_close(ctx_, array2);
-    REQUIRE(rc == TILEDB_OK);
+    REQUIRE(
+        rc == TILEDB_OK);  // Array not opened successfully, closing is a noop
     rc = tiledb_array_close(ctx_, array);
     REQUIRE(rc == TILEDB_OK);
 
@@ -954,7 +985,7 @@ TEST_CASE_METHOD(
   rc = tiledb_vfs_ls(
       ctx_,
       vfs_,
-      array_name.c_str(),
+      get_commit_dir(array_name).c_str(),
       &get_fragment_timestamps,
       &fragment_timestamps);
   CHECK(rc == TILEDB_OK);
@@ -1792,106 +1823,6 @@ TEST_CASE_METHOD(
   remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
 }
 
-TEST_CASE_METHOD(
-    ArrayFx,
-    "C API: Test array with no filelocks",
-    "[capi][array][array-no-filelocks]") {
-  // TODO: refactor for each supported FS.
-  std::string temp_dir = fs_vec_[0]->temp_dir();
-
-  std::string array_name = temp_dir + "array-no-filelocks";
-
-  // Create new TileDB context with file lock config disabled, rest the same.
-  tiledb_ctx_free(&ctx_);
-  tiledb_vfs_free(&vfs_);
-
-  tiledb_config_t* cfg = nullptr;
-  tiledb_error_t* err = nullptr;
-  REQUIRE(tiledb_config_alloc(&cfg, &err) == TILEDB_OK);
-  REQUIRE(err == nullptr);
-  REQUIRE(
-      tiledb_config_set(cfg, "vfs.file.enable_filelocks", "false", &err) ==
-      TILEDB_OK);
-  REQUIRE(err == nullptr);
-
-  REQUIRE(vfs_test_init(fs_vec_, &ctx_, &vfs_, cfg).ok());
-
-  tiledb_config_free(&cfg);
-
-  create_temp_dir(temp_dir);
-
-  create_dense_vector(array_name);
-
-  // Prepare cell buffers
-  int buffer_a1[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
-  uint64_t buffer_a1_size = sizeof(buffer_a1);
-
-  // Open array
-  int64_t subarray[] = {1, 10};
-  tiledb_array_t* array;
-  int rc = tiledb_array_alloc(ctx_, array_name.c_str(), &array);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
-  CHECK(rc == TILEDB_OK);
-
-  // Submit query
-  tiledb_query_t* query;
-  rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_data_buffer(
-      ctx_, query, "a", buffer_a1, &buffer_a1_size);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_submit(ctx_, query);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_finalize(ctx_, query);
-  CHECK(rc == TILEDB_OK);
-
-  // Close array and clean up
-  rc = tiledb_array_close(ctx_, array);
-  CHECK(rc == TILEDB_OK);
-  tiledb_array_free(&array);
-  tiledb_query_free(&query);
-
-  int buffer_read[10];
-  uint64_t buffer_read_size = sizeof(buffer_read);
-
-  // Open array
-  rc = tiledb_array_alloc(ctx_, array_name.c_str(), &array);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_array_open(ctx_, array, TILEDB_READ);
-  CHECK(rc == TILEDB_OK);
-
-  // Submit query
-  rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_data_buffer(
-      ctx_, query, "a", buffer_read, &buffer_read_size);
-  CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_submit(ctx_, query);
-  CHECK(rc == TILEDB_OK);
-
-  // Close array and clean up
-  rc = tiledb_array_close(ctx_, array);
-  CHECK(rc == TILEDB_OK);
-  tiledb_array_free(&array);
-  tiledb_query_free(&query);
-
-  // Check correctness
-  int buffer_read_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
-  CHECK(!std::memcmp(buffer_read, buffer_read_c, sizeof(buffer_read_c)));
-  CHECK(buffer_read_size == sizeof(buffer_read_c));
-
-  remove_temp_dir(temp_dir);
-}
-
 TEST_CASE_METHOD(
     ArrayFx,
     "C API: Test query errors, getting subarray info from write queries in "
@@ -2102,4 +2033,139 @@ TEST_CASE_METHOD(
   tiledb_query_free(&query);
 
   remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+}
+
+TEST_CASE_METHOD(
+    ArrayFx, "Test array serialization", "[array][serialization]") {
+#ifdef TILEDB_SERIALIZATION
+  SupportedFsLocal local_fs;
+  std::string array_name =
+      local_fs.file_prefix() + local_fs.temp_dir() + "array_serialization";
+  create_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+
+  create_dense_array(array_name);
+
+  // Open array
+  tiledb_array_t* array;
+  int rc = tiledb_array_alloc(ctx_, array_name.c_str(), &array);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_open(ctx_, array, TILEDB_READ);
+  REQUIRE(rc == TILEDB_OK);
+
+  int32_t a[4];
+  uint64_t a_size = sizeof(a);
+
+  // Prepare query
+  tiledb_query_t* query;
+  rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
+  CHECK(rc == TILEDB_OK);
+
+  int64_t subarray[] = {2, 3, 4, 5};
+  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
+
+  rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_submit(ctx_, query);
+  CHECK(rc == TILEDB_OK);
+
+  // Close array
+  rc = tiledb_array_close(ctx_, array);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Reopen array in WRITE mode
+  rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Write metadata
+  int32_t v = 5;
+  rc = tiledb_array_put_metadata(ctx_, array, "aaa", TILEDB_INT32, 1, &v);
+  CHECK(rc == TILEDB_OK);
+  float f[] = {1.1f, 1.2f};
+  rc = tiledb_array_put_metadata(ctx_, array, "bb", TILEDB_FLOAT32, 2, f);
+  CHECK(rc == TILEDB_OK);
+
+  // Serialize array and deserialize into new_array
+  tiledb_array_t* new_array = nullptr;
+  array_serialize_wrapper(array, new_array);
+
+  // Close array and clean up
+  rc = tiledb_array_close(ctx_, array);
+  CHECK(rc == TILEDB_OK);
+  tiledb_array_free(&array);
+  tiledb_query_free(&query);
+
+  rc = tiledb_array_alloc(ctx_, array_name.c_str(), &new_array);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_open(ctx_, new_array, TILEDB_READ);
+  REQUIRE(rc == TILEDB_OK);
+
+  int is_empty;
+  uint64_t domain[4];
+  rc = tiledb_array_get_non_empty_domain(ctx_, new_array, domain, &is_empty);
+  CHECK(rc == TILEDB_OK);
+  CHECK(is_empty == 1);
+
+  // Close new_array
+  rc = tiledb_array_close(ctx_, new_array);
+  CHECK(rc == TILEDB_OK);
+  tiledb_array_free(&new_array);
+
+  /** Validate metadata. **/
+  // Open new_array in read mode
+  rc = tiledb_array_alloc(ctx_, array_name.c_str(), &new_array);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_array_open(ctx_, new_array, TILEDB_READ);
+  REQUIRE(rc == TILEDB_OK);
+
+  // Read
+  const void* v_r;
+  tiledb_datatype_t v_type;
+  uint32_t v_num;
+  rc = tiledb_array_get_metadata(ctx_, new_array, "aaa", &v_type, &v_num, &v_r);
+  CHECK(rc == TILEDB_OK);
+  CHECK(v_type == TILEDB_INT32);
+  CHECK(v_num == 1);
+  CHECK(*((const int32_t*)v_r) == 5);
+
+  rc = tiledb_array_get_metadata(ctx_, new_array, "bb", &v_type, &v_num, &v_r);
+  CHECK(rc == TILEDB_OK);
+  CHECK(v_type == TILEDB_FLOAT32);
+  CHECK(v_num == 2);
+  CHECK(((const float*)v_r)[0] == 1.1f);
+  CHECK(((const float*)v_r)[1] == 1.2f);
+
+  uint64_t num = 0;
+  rc = tiledb_array_get_metadata_num(ctx_, new_array, &num);
+  CHECK(rc == TILEDB_OK);
+  CHECK(num == 2);
+
+  const char* key;
+  uint32_t key_len;
+  rc = tiledb_array_get_metadata_from_index(
+      ctx_, new_array, 1, &key, &key_len, &v_type, &v_num, &v_r);
+  CHECK(rc == TILEDB_OK);
+  CHECK(v_type == TILEDB_FLOAT32);
+  CHECK(v_num == 2);
+  CHECK(((const float*)v_r)[0] == 1.1f);
+  CHECK(((const float*)v_r)[1] == 1.2f);
+  CHECK(key_len == strlen("bb"));
+  CHECK(!strncmp(key, "bb", strlen("bb")));
+
+  // Check has_key
+  int32_t has_key = 0;
+  rc = tiledb_array_has_metadata_key(ctx_, new_array, "bb", &v_type, &has_key);
+  CHECK(rc == TILEDB_OK);
+  CHECK(v_type == TILEDB_FLOAT32);
+  CHECK(has_key == 1);
+
+  // Close array
+  rc = tiledb_array_close(ctx_, new_array);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_array_free(&new_array);
+
+  remove_temp_dir(local_fs.file_prefix() + local_fs.temp_dir());
+#endif
 }
\ No newline at end of file
