@@ -47,6 +47,9 @@ struct AsyncFx {
   // TileDB context
   tiledb_ctx_t* ctx_;
 
+  // separate subarray prepared 'outside' of a query
+  bool use_external_subarray_ = false;
+
   // Constructors/destructors
   AsyncFx();
   ~AsyncFx();
@@ -304,25 +307,39 @@ void AsyncFx::write_dense_async() {
       ctx_, query, attributes[2], buffers[3], &buffer_sizes[3]);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query asynchronously
-  int callback_made = 0;
-  rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
-  CHECK(rc == TILEDB_OK);
-
-  // Wait for query to complete
-  tiledb_query_status_t status;
-  do {
-    rc = tiledb_query_get_status(ctx_, query, &status);
+  auto proc_query = [&]() -> void {
+    // Submit query asynchronously
+    int callback_made = 0;
+    rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
     CHECK(rc == TILEDB_OK);
-  } while (status != TILEDB_COMPLETED);
 
-  // Finalize query
-  rc = tiledb_query_finalize(ctx_, query);
-  CHECK(rc == TILEDB_OK);
+    // Wait for query to complete
+    tiledb_query_status_t status;
+    do {
+      rc = tiledb_query_get_status(ctx_, query, &status);
+      CHECK(rc == TILEDB_OK);
+    } while (status != TILEDB_COMPLETED);
 
-  // Check correct execution of callback
-  CHECK(callback_made == 1);
+    // Finalize query
+    rc = tiledb_query_finalize(ctx_, query);
+    CHECK(rc == TILEDB_OK);
 
+    // Check correct execution of callback
+    CHECK(callback_made == 1);
+  };
+  if (!use_external_subarray_) {
+    proc_query();
+  } else {
+    tiledb_subarray_t* query_subarray;
+    rc = tiledb_query_get_subarray_t(ctx_, query, &query_subarray);
+    CHECK(rc == TILEDB_OK);
+    rc = tiledb_query_set_subarray_t(ctx_, query, query_subarray);
+    CHECK(rc == TILEDB_OK);
+
+    proc_query();
+
+    tiledb_subarray_free(&query_subarray);
+  }
   // Close array
   rc = tiledb_array_close(ctx_, array);
   CHECK(rc == TILEDB_OK);
@@ -401,29 +418,43 @@ void AsyncFx::write_sparse_async() {
       ctx_, query, attributes[4], buffers[5], &buffer_sizes[4]);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query asynchronously
-  int callback_made = 0;
-  rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
-  CHECK(rc == TILEDB_OK);
-
-  // Wait for query to complete
-  tiledb_query_status_t status;
-  do {
-    rc = tiledb_query_get_status(ctx_, query, &status);
+  auto proc_query = [&]() -> void {
+    // Submit query asynchronously
+    int callback_made = 0;
+    rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
     CHECK(rc == TILEDB_OK);
-  } while (status != TILEDB_COMPLETED);
 
-  // Finalize query
-  rc = tiledb_query_finalize(ctx_, query);
-  CHECK(rc == TILEDB_OK);
+    if (rc == TILEDB_OK) {
+      // Wait for query to complete
+      tiledb_query_status_t status;
+      do {
+        rc = tiledb_query_get_status(ctx_, query, &status);
+        CHECK(rc == TILEDB_OK);
+      } while (status != TILEDB_COMPLETED);
 
+      // Finalize query
+      rc = tiledb_query_finalize(ctx_, query);
+      CHECK(rc == TILEDB_OK);
+      // Check correct execution of callback
+      CHECK(callback_made == 1);
+    }
+  };
+  if (!use_external_subarray_) {
+    proc_query();
+  } else {
+    tiledb_subarray_t* query_subarray;
+    tiledb_query_get_subarray_t(ctx_, query, &query_subarray);
+    rc = tiledb_query_set_subarray_t(ctx_, query, query_subarray);
+    CHECK(rc == TILEDB_OK);
+
+    proc_query();
+
+    tiledb_subarray_free(&query_subarray);
+  }
   // Close array
   rc = tiledb_array_close(ctx_, array);
   CHECK(rc == TILEDB_OK);
 
-  // Check correct execution of callback
-  CHECK(callback_made == 1);
-
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
@@ -498,44 +529,62 @@ void AsyncFx::write_sparse_async_cancelled() {
       ctx_, query, attributes[4], buffers[5], &buffer_sizes[4]);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query asynchronously
-  int callback_made = 0;
-  rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
-  CHECK(rc == TILEDB_OK);
-
-  // Cancel it immediately, which sometimes in this test is fast enough to
-  // cancel it and sometimes not.
-  rc = tiledb_ctx_cancel_tasks(ctx_);
-  CHECK(rc == TILEDB_OK);
-
-  // Check query status
-  tiledb_query_status_t status;
-  do {
-    rc = tiledb_query_get_status(ctx_, query, &status);
-    CHECK(rc == TILEDB_OK);
-  } while (status != TILEDB_COMPLETED && status != TILEDB_FAILED);
-  CHECK((status == TILEDB_COMPLETED || status == TILEDB_FAILED));
-
-  // If the query completed, check the callback was made.
-  CHECK(callback_made == (status == TILEDB_COMPLETED ? 1 : 0));
-
-  // If it failed, run it again.
-  if (status == TILEDB_FAILED) {
+  auto proc_query = [&]() -> void {
+    // Submit query asynchronously
+    int callback_made = 0;
     rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
     CHECK(rc == TILEDB_OK);
-    do {
-      rc = tiledb_query_get_status(ctx_, query, &status);
+
+    tiledb_query_status_t status = TILEDB_FAILED;
+    if (rc == TILEDB_OK) {
+      // Cancel it immediately, which sometimes in this test is fast enough to
+      // cancel it and sometimes not.
+      rc = tiledb_ctx_cancel_tasks(ctx_);
       CHECK(rc == TILEDB_OK);
-    } while (status != TILEDB_COMPLETED && status != TILEDB_FAILED);
+
+      // Check query status
+      do {
+        rc = tiledb_query_get_status(ctx_, query, &status);
+        CHECK(rc == TILEDB_OK);
+      } while (status != TILEDB_COMPLETED && status != TILEDB_FAILED);
+      CHECK((status == TILEDB_COMPLETED || status == TILEDB_FAILED));
+
+      // If the query completed, check the callback was made.
+      CHECK(callback_made == (status == TILEDB_COMPLETED ? 1 : 0));
+    }
+
+    // If it failed, run it again.
+    if (status == TILEDB_FAILED) {
+      rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
+      CHECK(rc == TILEDB_OK);
+      if (rc == TILEDB_OK) {
+        do {
+          rc = tiledb_query_get_status(ctx_, query, &status);
+          CHECK(rc == TILEDB_OK);
+        } while (status != TILEDB_COMPLETED && status != TILEDB_FAILED);
+      }
+    }
+
+    CHECK(status == TILEDB_COMPLETED);
+    CHECK(callback_made == 1);
+
+    // Finalize query whether successfully submitted or not
+    rc = tiledb_query_finalize(ctx_, query);
+    CHECK(rc == TILEDB_OK);
+  };
+  if (!use_external_subarray_) {
+    proc_query();
+  } else {
+    tiledb_subarray_t* query_subarray;
+    tiledb_query_get_subarray_t(ctx_, query, &query_subarray);
+    rc = tiledb_query_set_subarray_t(ctx_, query, query_subarray);
+    CHECK(rc == TILEDB_OK);
+
+    proc_query();
+
+    tiledb_subarray_free(&query_subarray);
   }
 
-  CHECK(status == TILEDB_COMPLETED);
-  CHECK(callback_made == 1);
-
-  // Finalize query
-  rc = tiledb_query_finalize(ctx_, query);
-  CHECK(rc == TILEDB_OK);
-
   // Close array
   rc = tiledb_array_close(ctx_, array);
   CHECK(rc == TILEDB_OK);
@@ -587,46 +636,61 @@ void AsyncFx::read_dense_async() {
       ctx_, query, "a3", buffer_a3, &buffer_a3_size);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query with callback
-  int callback_made = 0;
-  rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
-  CHECK(rc == TILEDB_OK);
+  auto proc_query = [&]() -> void {
+    // Submit query with callback
+    int callback_made = 0;
+    rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
+    CHECK(rc == TILEDB_OK);
 
-  // Wait for the query to complete
-  tiledb_query_status_t status;
-  do {
-    tiledb_query_get_status(ctx_, query, &status);
-  } while (status != TILEDB_COMPLETED);
+    if (rc == TILEDB_OK) {
+      // Wait for the query to complete
+      tiledb_query_status_t status;
+      do {
+        tiledb_query_get_status(ctx_, query, &status);
+      } while (status != TILEDB_COMPLETED);
 
-  // Finalize query
-  rc = tiledb_query_finalize(ctx_, query);
-  CHECK(rc == TILEDB_OK);
+      // Finalize query
+      rc = tiledb_query_finalize(ctx_, query);
+      CHECK(rc == TILEDB_OK);
 
-  // Check correct execution of callback
-  CHECK(callback_made == 1);
+      // Check correct execution of callback
+      CHECK(callback_made == 1);
+    }
 
-  // Correct buffers
-  int c_buffer_a1[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
-  uint64_t c_buffer_a2_off[] = {
-      0, 1, 3, 6, 10, 11, 13, 16, 20, 21, 23, 26, 30, 31, 33, 36};
-  char c_buffer_a2_val[] =
-      "abbcccdddd"
-      "effggghhhh"
-      "ijjkkkllll"
-      "mnnooopppp";
-  float c_buffer_a3[] = {
-      0.1f,  0.2f,  1.1f,  1.2f,  2.1f,  2.2f,  3.1f,  3.2f,
-      4.1f,  4.2f,  5.1f,  5.2f,  6.1f,  6.2f,  7.1f,  7.2f,
-      8.1f,  8.2f,  9.1f,  9.2f,  10.1f, 10.2f, 11.1f, 11.2f,
-      12.1f, 12.2f, 13.1f, 13.2f, 14.1f, 14.2f, 15.1f, 15.2f,
+    // Correct buffers
+    int c_buffer_a1[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15};
+    uint64_t c_buffer_a2_off[] = {
+        0, 1, 3, 6, 10, 11, 13, 16, 20, 21, 23, 26, 30, 31, 33, 36};
+    char c_buffer_a2_val[] =
+        "abbcccdddd"
+        "effggghhhh"
+        "ijjkkkllll"
+        "mnnooopppp";
+    float c_buffer_a3[] = {
+        0.1f,  0.2f,  1.1f,  1.2f,  2.1f,  2.2f,  3.1f,  3.2f,
+        4.1f,  4.2f,  5.1f,  5.2f,  6.1f,  6.2f,  7.1f,  7.2f,
+        8.1f,  8.2f,  9.1f,  9.2f,  10.1f, 10.2f, 11.1f, 11.2f,
+        12.1f, 12.2f, 13.1f, 13.2f, 14.1f, 14.2f, 15.1f, 15.2f,
+    };
+
+    // Check buffers
+    CHECK(!memcmp(buffer_a1, c_buffer_a1, sizeof(c_buffer_a1)));
+    CHECK(!memcmp(buffer_a2_off, c_buffer_a2_off, sizeof(c_buffer_a2_off)));
+    CHECK(!memcmp(buffer_a2_val, c_buffer_a2_val, sizeof(c_buffer_a2_val) - 1));
+    CHECK(!memcmp(buffer_a3, c_buffer_a3, sizeof(c_buffer_a3)));
   };
+  if (!use_external_subarray_) {
+    proc_query();
+  } else {
+    tiledb_subarray_t* query_subarray;
+    tiledb_query_get_subarray_t(ctx_, query, &query_subarray);
+    rc = tiledb_query_set_subarray_t(ctx_, query, query_subarray);
+    CHECK(rc == TILEDB_OK);
 
-  // Check buffers
-  CHECK(!memcmp(buffer_a1, c_buffer_a1, sizeof(c_buffer_a1)));
-  CHECK(!memcmp(buffer_a2_off, c_buffer_a2_off, sizeof(c_buffer_a2_off)));
-  CHECK(!memcmp(buffer_a2_val, c_buffer_a2_val, sizeof(c_buffer_a2_val) - 1));
-  CHECK(!memcmp(buffer_a3, c_buffer_a3, sizeof(c_buffer_a3)));
+    proc_query();
 
+    tiledb_subarray_free(&query_subarray);
+  }
   // Close array
   rc = tiledb_array_close(ctx_, array);
   CHECK(rc == TILEDB_OK);
@@ -689,56 +753,75 @@ void AsyncFx::read_sparse_async() {
       ctx_, query, "d2", buffer_coords_dim2, &buffer_coords_dim2_size);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query with callback
-  int callback_made = 0;
-  rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
-  CHECK(rc == TILEDB_OK);
+  auto proc_query = [&]() -> void {
+    // Submit query with callback
+    int callback_made = 0;
+    rc = tiledb_query_submit_async(ctx_, query, callback, &callback_made);
+    CHECK(rc == TILEDB_OK);
 
-  // Wait for the query to complete
-  tiledb_query_status_t status;
-  do {
-    tiledb_query_get_status(ctx_, query, &status);
-  } while (status != TILEDB_COMPLETED);
+    // Wait for the query to complete
+    tiledb_query_status_t status;
+    do {
+      tiledb_query_get_status(ctx_, query, &status);
+    } while (status != TILEDB_COMPLETED);
 
-  // Finalize query
-  rc = tiledb_query_finalize(ctx_, query);
-  CHECK(rc == TILEDB_OK);
+    // Finalize query
+    rc = tiledb_query_finalize(ctx_, query);
+    CHECK(rc == TILEDB_OK);
 
-  // Check correct execution of callback
-  CHECK(callback_made == 1);
+    // Check correct execution of callback
+    CHECK(callback_made == 1);
 
-  // Correct buffers
-  int c_buffer_a1[] = {0, 1, 2, 3, 4, 5, 6, 7};
-  uint64_t c_buffer_a2_off[] = {0, 1, 3, 6, 10, 11, 13, 16};
-  char c_buffer_a2_val[] = "abbcccddddeffggghhhh";
-  float c_buffer_a3[] = {0.1f,
-                         0.2f,
-                         1.1f,
-                         1.2f,
-                         2.1f,
-                         2.2f,
-                         3.1f,
-                         3.2f,
-                         4.1f,
-                         4.2f,
-                         5.1f,
-                         5.2f,
-                         6.1f,
-                         6.2f,
-                         7.1f,
-                         7.2f};
-  uint64_t c_buffer_coords_dim1[] = {1, 1, 1, 2, 3, 4, 3, 3};
-  uint64_t c_buffer_coords_dim2[] = {1, 2, 4, 3, 1, 2, 3, 4};
+    // Correct buffers
+    int c_buffer_a1[] = {0, 1, 2, 3, 4, 5, 6, 7};
+    uint64_t c_buffer_a2_off[] = {0, 1, 3, 6, 10, 11, 13, 16};
+    char c_buffer_a2_val[] = "abbcccddddeffggghhhh";
+    float c_buffer_a3[] = {0.1f,
+                           0.2f,
+                           1.1f,
+                           1.2f,
+                           2.1f,
+                           2.2f,
+                           3.1f,
+                           3.2f,
+                           4.1f,
+                           4.2f,
+                           5.1f,
+                           5.2f,
+                           6.1f,
+                           6.2f,
+                           7.1f,
+                           7.2f};
+    uint64_t c_buffer_coords_dim1[] = {1, 1, 1, 2, 3, 4, 3, 3};
+    uint64_t c_buffer_coords_dim2[] = {1, 2, 4, 3, 1, 2, 3, 4};
 
-  // Check buffers
-  CHECK(!memcmp(buffer_a1, c_buffer_a1, sizeof(c_buffer_a1)));
-  CHECK(!memcmp(buffer_a2_off, c_buffer_a2_off, sizeof(c_buffer_a2_off)));
-  CHECK(!memcmp(buffer_a2_val, c_buffer_a2_val, sizeof(c_buffer_a2_val) - 1));
-  CHECK(!memcmp(buffer_a3, c_buffer_a3, sizeof(c_buffer_a3)));
-  CHECK(!memcmp(
-      buffer_coords_dim1, c_buffer_coords_dim1, sizeof(c_buffer_coords_dim1)));
-  CHECK(!memcmp(
-      buffer_coords_dim2, c_buffer_coords_dim2, sizeof(c_buffer_coords_dim2)));
+    // Check buffers
+    CHECK(!memcmp(buffer_a1, c_buffer_a1, sizeof(c_buffer_a1)));
+    CHECK(!memcmp(buffer_a2_off, c_buffer_a2_off, sizeof(c_buffer_a2_off)));
+    CHECK(!memcmp(buffer_a2_val, c_buffer_a2_val, sizeof(c_buffer_a2_val) - 1));
+    CHECK(!memcmp(buffer_a3, c_buffer_a3, sizeof(c_buffer_a3)));
+    CHECK(!memcmp(
+        buffer_coords_dim1,
+        c_buffer_coords_dim1,
+        sizeof(c_buffer_coords_dim1)));
+    CHECK(!memcmp(
+        buffer_coords_dim2,
+        c_buffer_coords_dim2,
+        sizeof(c_buffer_coords_dim2)));
+  };
+
+  if (!use_external_subarray_) {
+    proc_query();
+  } else {
+    tiledb_subarray_t* query_subarray;
+    tiledb_query_get_subarray_t(ctx_, query, &query_subarray);
+    rc = tiledb_query_set_subarray_t(ctx_, query, query_subarray);
+    CHECK(rc == TILEDB_OK);
+
+    proc_query();
+
+    tiledb_subarray_free(&query_subarray);
+  }
 
   // Close array
   rc = tiledb_array_close(ctx_, array);
@@ -778,6 +861,12 @@ bool AsyncFx::is_array(const std::string& array_name) {
 
 TEST_CASE_METHOD(
     AsyncFx, "C API: Test dense async", "[capi][async][dense-async]") {
+  SECTION("- No outside subarray") {
+    use_external_subarray_ = false;
+  }
+  SECTION("- outside subarray") {
+    use_external_subarray_ = true;
+  }
   remove_dense_array();
   create_dense_array();
   write_dense_async();
@@ -787,6 +876,12 @@ TEST_CASE_METHOD(
 
 TEST_CASE_METHOD(
     AsyncFx, "C API: Test sparse async", "[capi][async][sparse-async]") {
+  SECTION("- No outside subarray") {
+    use_external_subarray_ = false;
+  }
+  SECTION("- outside subarray") {
+    use_external_subarray_ = true;
+  }
   remove_sparse_array();
   create_sparse_array();
   write_sparse_async();
@@ -796,6 +891,12 @@ TEST_CASE_METHOD(
 
 TEST_CASE_METHOD(
     AsyncFx, "C API: Test async cancellation", "[capi][async][cancel]") {
+  SECTION("- No outside subarray") {
+    use_external_subarray_ = false;
+  }
+  SECTION("- outside subarray") {
+    use_external_subarray_ = true;
+  }
   remove_sparse_array();
   create_sparse_array();
   write_sparse_async_cancelled();
