@@ -34,6 +34,7 @@
 #include "test/src/helpers.h"
 #include "test/src/vfs_helpers.h"
 #ifdef _WIN32
+#include "tiledb/sm/filesystem/path_win.h"
 #include "tiledb/sm/filesystem/win.h"
 #else
 #include "tiledb/sm/filesystem/posix.h"
@@ -394,8 +395,9 @@ TEST_CASE_METHOD(
     // `VFS::ls(...)` returns `file:///` URIs instead of Windows paths.
     SupportedFsLocal windows_fs;
     std::string temp_dir = windows_fs.file_prefix() + windows_fs.temp_dir();
-    golden_walk = get_golden_walk(tiledb::sm::Win::uri_from_path(temp_dir));
-    golden_ls = get_golden_ls(tiledb::sm::Win::uri_from_path(temp_dir));
+    golden_walk =
+        get_golden_walk(tiledb::sm::path_win::uri_from_path(temp_dir));
+    golden_ls = get_golden_ls(tiledb::sm::path_win::uri_from_path(temp_dir));
 #else
     SupportedFsLocal posix_fs;
     std::string temp_dir = posix_fs.file_prefix() + posix_fs.temp_dir();
