@@ -35,6 +35,7 @@
 #include "tiledb/sm/c_api/tiledb.h"
 #include "tiledb/sm/misc/utils.h"
 #ifdef _WIN32
+#include "tiledb/sm/filesystem/path_win.h"
 #include "tiledb/sm/filesystem/win.h"
 #else
 #include "tiledb/sm/filesystem/posix.h"
@@ -879,10 +880,10 @@ void VFSFx::check_ls(const std::string& path) {
 
 #ifdef _WIN32
   // Normalization only for Windows
-  file = tiledb::sm::Win::uri_from_path(file);
-  file2 = tiledb::sm::Win::uri_from_path(file2);
-  subdir = tiledb::sm::Win::uri_from_path(subdir);
-  subdir2 = tiledb::sm::Win::uri_from_path(subdir2);
+  file = tiledb::sm::path_win::uri_from_path(file);
+  file2 = tiledb::sm::path_win::uri_from_path(file2);
+  subdir = tiledb::sm::path_win::uri_from_path(subdir);
+  subdir2 = tiledb::sm::path_win::uri_from_path(subdir2);
 #endif
 
   // Check results
