@@ -52,9 +52,15 @@ TEST_CASE("C++ API: Config", "[cppapi][config]") {
 
   auto readInvalidKey = [&config]() { std::string result2 = config["bar"]; };
   REQUIRE_THROWS_AS(readInvalidKey(), tiledb::TileDBError);
+
+  bool contains = config.contains("foo");
+  CHECK(contains == true);
+
+  contains = config.contains("bar");
+  CHECK(contains == false);
 }
 
-TEST_CASE("C++ API: Config iterator", "[cppapi][cppapi-config]") {
+TEST_CASE("C++ API: Config iterator", "[cppapi][config]") {
   tiledb::Config config;
   std::vector<std::string> names;
   for (auto it = config.begin("vfs"), ite = config.end(); it != ite; ++it) {
@@ -64,7 +70,7 @@ TEST_CASE("C++ API: Config iterator", "[cppapi][cppapi-config]") {
   CHECK(names.size() == 58);
 }
 
-TEST_CASE("C++ API: Config Environment Variables", "[cppapi][cppapi-config]") {
+TEST_CASE("C++ API: Config Environment Variables", "[cppapi][config]") {
   tiledb::Config config;
   auto readInvalidKey = [&config]() { std::string result1 = config["foo"]; };
   REQUIRE_THROWS_AS(readInvalidKey(), tiledb::TileDBError);
@@ -88,7 +94,7 @@ TEST_CASE("C++ API: Config Environment Variables", "[cppapi][cppapi-config]") {
 
 TEST_CASE(
     "C++ API: Config Environment Variables Default Override",
-    "[cppapi][cppapi-config]") {
+    "[cppapi][config]") {
   tiledb::Config config;
   const std::string key = "sm.io_concurrency_level";
 
@@ -114,7 +120,7 @@ TEST_CASE(
   CHECK(result3 == value3);
 }
 
-TEST_CASE("C++ API: Config Equality", "[cppapi][cppapi-config]") {
+TEST_CASE("C++ API: Config Equality", "[cppapi][config]") {
   // Check for equality
   tiledb::Config config1;
   config1["foo"] = "bar";
@@ -130,8 +136,7 @@ TEST_CASE("C++ API: Config Equality", "[cppapi][cppapi-config]") {
 }
 
 #ifdef TILEDB_SERIALIZATION
-TEST_CASE(
-    "C++ API: Config Serialization", "[cppapi][cppapi-config][serialization]") {
+TEST_CASE("C++ API: Config Serialization", "[cppapi][config][serialization]") {
   // this variable is parameterized below, but we initialize
   // here to avoid warning/error on MSVC
   //   C4701: potentially uninitialized local variable 'format'
