@@ -36,13 +36,15 @@
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/filter_type.h"
 #include "tiledb/sm/filter/compression_filter.h"
-#include "tiledb/sm/misc/utils.h"
+#include "tiledb/sm/misc/parse_argument.h"
+#include "tiledb/type/range/range.h"
 
 #include <cassert>
 #include <iostream>
 #include <sstream>
 
 using namespace tiledb::common;
+using namespace tiledb::type;
 
 namespace tiledb {
 namespace sm {
@@ -64,6 +66,23 @@ Attribute::Attribute(
   set_default_fill_value();
 }
 
+Attribute::Attribute(
+    const std::string& name,
+    Datatype type,
+    bool nullable,
+    uint32_t cell_val_num,
+    const FilterPipeline& filter_pipeline,
+    const ByteVecValue& fill_value,
+    uint8_t fill_value_validity)
+    : cell_val_num_(cell_val_num)
+    , nullable_(nullable)
+    , filters_(filter_pipeline)
+    , name_(name)
+    , type_(type)
+    , fill_value_(fill_value)
+    , fill_value_validity_(fill_value_validity) {
+}
+
 Attribute::Attribute(const Attribute* attr) {
   assert(attr != nullptr);
   name_ = attr->name();
@@ -92,47 +111,73 @@ unsigned int Attribute::cell_val_num() const {
   return cell_val_num_;
 }
 
-Status Attribute::deserialize(ConstBuffer* buff, const uint32_t version) {
+tuple<Status, optional<Attribute>> Attribute::deserialize(
+    ConstBuffer* buff, const uint32_t version) {
   // Load attribute name
   uint32_t attribute_name_size;
-  RETURN_NOT_OK(buff->read(&attribute_name_size, sizeof(uint32_t)));
-  name_.resize(attribute_name_size);
-  RETURN_NOT_OK(buff->read(&name_[0], attribute_name_size));
+  RETURN_NOT_OK_TUPLE(
+      buff->read(&attribute_name_size, sizeof(uint32_t)), nullopt);
+
+  std::string name;
+  name.resize(attribute_name_size);
+  RETURN_NOT_OK_TUPLE(buff->read(&name[0], attribute_name_size), nullopt);
 
   // Load type
   uint8_t type;
-  RETURN_NOT_OK(buff->read(&type, sizeof(uint8_t)));
-  type_ = (Datatype)type;
+  RETURN_NOT_OK_TUPLE(buff->read(&type, sizeof(uint8_t)), nullopt);
 
-  // Load cell_val_num_
-  RETURN_NOT_OK(buff->read(&cell_val_num_, sizeof(uint32_t)));
+  Datatype datatype = static_cast<Datatype>(type);
+
+  // Load cell_val_num
+  uint32_t cell_val_num;
+  RETURN_NOT_OK_TUPLE(buff->read(&cell_val_num, sizeof(uint32_t)), nullopt);
 
   // Load filter pipeline
-  RETURN_NOT_OK(filters_.deserialize(buff));
+  auto&& [st_filterpipeline, filterpipeline]{
+      FilterPipeline::deserialize(buff, version)};
+  if (!st_filterpipeline.ok()) {
+    return {st_filterpipeline, nullopt};
+  }
 
   // Load fill value
+  uint64_t fill_value_size = 0;
+  ByteVecValue fill_value;
   if (version >= 6) {
-    uint64_t fill_value_size = 0;
-    RETURN_NOT_OK(buff->read(&fill_value_size, sizeof(uint64_t)));
+    RETURN_NOT_OK_TUPLE(
+        buff->read(&fill_value_size, sizeof(uint64_t)), nullopt);
     assert(fill_value_size > 0);
-    fill_value_.resize(fill_value_size);
-    fill_value_.shrink_to_fit();
-    RETURN_NOT_OK(buff->read(fill_value_.data(), fill_value_size));
+    fill_value.resize(fill_value_size);
+    fill_value.shrink_to_fit();
+    RETURN_NOT_OK_TUPLE(
+        buff->read(fill_value.data(), fill_value_size), nullopt);
   } else {
-    set_default_fill_value();
+    fill_value = default_fill_value(datatype, cell_val_num);
   }
 
   // Load nullable flag
-  if (version >= 7)
-    RETURN_NOT_OK(buff->read(&nullable_, sizeof(bool)));
+  bool nullable = false;
+  if (version >= 7) {
+    RETURN_NOT_OK_TUPLE(buff->read(&nullable, sizeof(bool)), nullopt);
+  }
 
   // Load validity fill value
-  if (version >= 7)
-    RETURN_NOT_OK(buff->read(&fill_value_validity_, sizeof(uint8_t)));
+  uint8_t fill_value_validity = 0;
+  if (version >= 7) {
+    RETURN_NOT_OK_TUPLE(
+        buff->read(&fill_value_validity, sizeof(uint8_t)), nullopt);
+  }
 
-  return Status::Ok();
+  return {Status::Ok(),
+          optional<Attribute>(
+              std::in_place,
+              name,
+              datatype,
+              nullable,
+              cell_val_num,
+              filterpipeline.value(),
+              fill_value,
+              fill_value_validity)};
 }
-
 void Attribute::dump(FILE* out) const {
   if (out == nullptr)
     out = stdout;
@@ -174,7 +219,7 @@ const std::string& Attribute::name() const {
 // fill_value (uint8_t[])
 // nullable (bool)
 // fill_value_validity (uint8_t)
-Status Attribute::serialize(Buffer* buff, const uint32_t version) {
+const Status Attribute::serialize(Buffer* buff, const uint32_t version) const {
   // Write attribute name
   auto attribute_name_size = (uint32_t)name_.size();
   RETURN_NOT_OK(buff->write(&attribute_name_size, sizeof(uint32_t)));
@@ -211,7 +256,7 @@ Status Attribute::serialize(Buffer* buff, const uint32_t version) {
 
 Status Attribute::set_cell_val_num(unsigned int cell_val_num) {
   if (type_ == Datatype::ANY)
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set number of values per cell; Attribute datatype `ANY` is "
         "always variable-sized"));
 
@@ -233,15 +278,27 @@ Status Attribute::get_nullable(bool* const nullable) {
 
 Status Attribute::set_filter_pipeline(const FilterPipeline* pipeline) {
   if (pipeline == nullptr)
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set filter pipeline to attribute; Pipeline cannot be null"));
 
   for (unsigned i = 0; i < pipeline->size(); ++i) {
     if (datatype_is_real(type_) &&
         pipeline->get_filter(i)->type() == FilterType::FILTER_DOUBLE_DELTA)
       return LOG_STATUS(
-          Status::AttributeError("Cannot set DOUBLE DELTA filter to a "
-                                 "dimension with a real datatype"));
+          Status_AttributeError("Cannot set DOUBLE DELTA filter to an "
+                                "attribute with a real datatype"));
+  }
+
+  if (type_ == Datatype::STRING_ASCII && var_size() && pipeline->size() > 1) {
+    if (pipeline->has_filter(FilterType::FILTER_RLE)) {
+      return LOG_STATUS(Status_AttributeError(
+          "RLE filter cannot be combined with other filters when applied to "
+          "variable length string attributes"));
+    } else if (pipeline->has_filter(FilterType::FILTER_DICTIONARY)) {
+      return LOG_STATUS(Status_AttributeError(
+          "Dictionary-encoding filter cannot be combined with other filters "
+          "when applied to variable length string attributes"));
+    }
   }
 
   filters_ = *pipeline;
@@ -255,22 +312,22 @@ void Attribute::set_name(const std::string& name) {
 
 Status Attribute::set_fill_value(const void* value, uint64_t size) {
   if (value == nullptr) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set fill value; Input value cannot be null"));
   }
 
   if (size == 0) {
-    return LOG_STATUS(Status::AttributeError(
-        "Cannot set fill value; Input size cannot be 0"));
+    return LOG_STATUS(
+        Status_AttributeError("Cannot set fill value; Input size cannot be 0"));
   }
 
   if (nullable()) {
     return LOG_STATUS(
-        Status::AttributeError("Cannot set fill value; Attribute is nullable"));
+        Status_AttributeError("Cannot set fill value; Attribute is nullable"));
   }
 
   if (!var_size() && size != cell_size()) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set fill value; Input size is not the same as cell size"));
   }
 
@@ -283,18 +340,18 @@ Status Attribute::set_fill_value(const void* value, uint64_t size) {
 
 Status Attribute::get_fill_value(const void** value, uint64_t* size) const {
   if (value == nullptr) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot get fill value; Input value cannot be null"));
   }
 
   if (size == nullptr) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot get fill value; Input size cannot be null"));
   }
 
   if (nullable()) {
     return LOG_STATUS(
-        Status::AttributeError("Cannot get fill value; Attribute is nullable"));
+        Status_AttributeError("Cannot get fill value; Attribute is nullable"));
   }
 
   *value = fill_value_.data();
@@ -306,22 +363,22 @@ Status Attribute::get_fill_value(const void** value, uint64_t* size) const {
 Status Attribute::set_fill_value(
     const void* value, uint64_t size, uint8_t valid) {
   if (value == nullptr) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set fill value; Input value cannot be null"));
   }
 
   if (size == 0) {
-    return LOG_STATUS(Status::AttributeError(
-        "Cannot set fill value; Input size cannot be 0"));
+    return LOG_STATUS(
+        Status_AttributeError("Cannot set fill value; Input size cannot be 0"));
   }
 
   if (!nullable()) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set fill value; Attribute is not nullable"));
   }
 
   if (!var_size() && size != cell_size()) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot set fill value; Input size is not the same as cell size"));
   }
 
@@ -336,17 +393,17 @@ Status Attribute::set_fill_value(
 Status Attribute::get_fill_value(
     const void** value, uint64_t* size, uint8_t* valid) const {
   if (value == nullptr) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot get fill value; Input value cannot be null"));
   }
 
   if (size == nullptr) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot get fill value; Input size cannot be null"));
   }
 
   if (!nullable()) {
-    return LOG_STATUS(Status::AttributeError(
+    return LOG_STATUS(Status_AttributeError(
         "Cannot get fill value; Attribute is not nullable"));
   }
 
@@ -398,6 +455,25 @@ void Attribute::set_default_fill_value() {
   fill_value_validity_ = 0;
 }
 
+ByteVecValue Attribute::default_fill_value(
+    Datatype datatype, uint32_t cell_val_num) {
+  ByteVecValue fillvalue;
+  auto fill_value = constants::fill_value(datatype);
+  auto fill_size = datatype_size(datatype);
+  if (cell_val_num == constants::var_num) {
+    cell_val_num = 1;
+  }
+  fillvalue.resize(cell_val_num * fill_size);
+  fillvalue.shrink_to_fit();
+  uint64_t offset = 0;
+  auto buff = (unsigned char*)fillvalue.data();
+  for (uint64_t i = 0; i < cell_val_num; ++i) {
+    std::memcpy(buff + offset, fill_value, fill_size);
+    offset += fill_size;
+  }
+  return fillvalue;
+}
+
 std::string Attribute::fill_value_str() const {
   std::string ret;
 
