@@ -33,11 +33,17 @@
 #ifndef TILEDB_ATTRIBUTE_H
 #define TILEDB_ATTRIBUTE_H
 
+#include "tiledb/common/common.h"
 #include "tiledb/common/status.h"
 #include "tiledb/sm/filter/filter_pipeline.h"
 #include "tiledb/sm/misc/types.h"
 
 using namespace tiledb::common;
+using namespace tiledb::type;
+
+namespace tiledb::type {
+class Range;
+}
 
 namespace tiledb {
 namespace sm {
@@ -69,6 +75,26 @@ class Attribute {
    */
   Attribute(const std::string& name, Datatype type, bool nullable = false);
 
+  /**
+   * Constructor.
+   *
+   * @param name The name of the attribute.
+   * @param type The type of the attribute.
+   * @param nullable The nullable of the attribute.
+   * @param cell_val_num The cell number of the attribute.
+   * @param filter_pipeline The filters of the attribute.
+   * @param fill_value The fill value of the attribute.
+   * @param fill_value_validity The validity of fill_value.
+   */
+  Attribute(
+      const std::string& name,
+      Datatype type,
+      bool nullable,
+      uint32_t cell_val_num,
+      const FilterPipeline& filter_pipeline,
+      const ByteVecValue& fill_value,
+      uint8_t fill_value_validity);
+
   /**
    * Constructor. It clones the input attribute.
    *
@@ -86,6 +112,9 @@ class Attribute {
   /** Copy-assignment operator. */
   DISABLE_COPY_ASSIGN(Attribute);
 
+  /** Move constructor. */
+  Attribute(Attribute&&) = default;
+
   /** Destructor. */
   ~Attribute();
 
@@ -107,9 +136,10 @@ class Attribute {
    *
    * @param buff The buffer to deserialize from.
    * @param version The format spec version.
-   * @return Status
+   * @return Status and Attribute
    */
-  Status deserialize(ConstBuffer* buff, uint32_t version);
+  static tuple<Status, optional<Attribute>> deserialize(
+      ConstBuffer* buff, uint32_t version);
 
   /** Dumps the attribute contents in ASCII form in the selected output. */
   void dump(FILE* out) const;
@@ -127,7 +157,7 @@ class Attribute {
    * @param version The format spec version.
    * @return Status
    */
-  Status serialize(Buffer* buff, uint32_t version);
+  const Status serialize(Buffer* buff, uint32_t version) const;
 
   /**
    * Sets the attribute number of values per cell. Note that if the attribute
@@ -205,6 +235,10 @@ class Attribute {
    */
   bool nullable() const;
 
+  /** The default fill value. */
+  static ByteVecValue default_fill_value(
+      Datatype datatype, uint32_t cell_val_num);
+
  private:
   /* ********************************* */
   /*         PRIVATE ATTRIBUTES        */
