@@ -57,7 +57,8 @@ typedef enum {
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_serialization_type_to_str(
-    tiledb_serialization_type_t serialization_type, const char** str);
+    tiledb_serialization_type_t serialization_type,
+    const char** str) TILEDB_NOEXCEPT;
 
 /**
  * Parses a serialization type from the given string.
@@ -67,12 +68,52 @@ TILEDB_EXPORT int32_t tiledb_serialization_type_to_str(
  * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
  */
 TILEDB_EXPORT int32_t tiledb_serialization_type_from_str(
-    const char* str, tiledb_serialization_type_t* serialization_type);
+    const char* str,
+    tiledb_serialization_type_t* serialization_type) TILEDB_NOEXCEPT;
 
 /* ****************************** */
 /*          Serialization         */
 /* ****************************** */
 
+/**
+ * Serializes the given array.
+ *
+ * @note The caller must free the returned `tiledb_buffer_t`.
+ *
+ * @param ctx The TileDB context.
+ * @param array The array to serialize.
+ * @param serialization_type Type of serialization to use
+ * @param client_side If set to 1, serialize from "client-side" perspective.
+ *    Else, "server-side."
+ * @param buffer Will be set to a newly allocated buffer containing the
+ *      serialized max buffer sizes.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_serialize_array(
+    tiledb_ctx_t* ctx,
+    const tiledb_array_t* array,
+    tiledb_serialization_type_t serialize_type,
+    int32_t client_side,
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
+
+/**
+ * Deserializes a new array from the given buffer.
+ *
+ * @param ctx The TileDB context.
+ * @param buffer Buffer to deserialize from
+ * @param serialization_type Type of serialization to use
+ * @param client_side If set to 1, deserialize from "client-side" perspective.
+ *    Else, "server-side."
+ * @param array Will be set to a newly allocated array.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_deserialize_array(
+    tiledb_ctx_t* ctx,
+    const tiledb_buffer_t* buffer,
+    tiledb_serialization_type_t serialize_type,
+    int32_t client_side,
+    tiledb_array_t** array) TILEDB_NOEXCEPT;
+
 /**
  * Serializes the given array schema.
  *
@@ -92,7 +133,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array_schema(
     const tiledb_array_schema_t* array_schema,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes a new array schema from the given buffer.
@@ -110,7 +151,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_schema(
     const tiledb_buffer_t* buffer,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_array_schema_t** array_schema);
+    tiledb_array_schema_t** array_schema) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the given array schema evolution.
@@ -131,7 +172,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array_schema_evolution(
     const tiledb_array_schema_evolution_t* array_schema_evolution,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes a new array schema evolution object from the given buffer.
@@ -150,7 +191,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_schema_evolution(
     const tiledb_buffer_t* buffer,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_array_schema_evolution_t** array_schema_evolution);
+    tiledb_array_schema_evolution_t** array_schema_evolution) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the given query.
@@ -175,7 +216,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_query(
     const tiledb_query_t* query,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_list_t** buffer_list);
+    tiledb_buffer_list_t** buffer_list) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes into an existing query from the given buffer.
@@ -196,7 +237,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_query(
     const tiledb_buffer_t* buffer,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_query_t* query);
+    tiledb_query_t* query) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the given non-empty domain information into the given buffer.
@@ -221,7 +262,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array_nonempty_domain(
     int32_t is_empty,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes non-empty domain information from the given buffer.
@@ -243,7 +284,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_nonempty_domain(
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
     void* nonempty_domain,
-    int32_t* is_empty);
+    int32_t* is_empty) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the given non-empty domain information into the given buffer.
@@ -264,7 +305,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array_non_empty_domain_all_dimensions(
     const tiledb_array_t* array,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes non-empty domain information from the given buffer.
@@ -282,7 +323,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_non_empty_domain_all_dimensions(
     tiledb_array_t* array,
     const tiledb_buffer_t* buffer,
     tiledb_serialization_type_t serialize_type,
-    int32_t client_side);
+    int32_t client_side) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the array max buffer sizes information into the given buffer.
@@ -302,7 +343,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array_max_buffer_sizes(
     const tiledb_array_t* array,
     const void* subarray,
     tiledb_serialization_type_t serialize_type,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the array metadata into the given buffer.
@@ -320,7 +361,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_array_metadata(
     tiledb_ctx_t* ctx,
     const tiledb_array_t* array,
     tiledb_serialization_type_t serialization_type,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Sets the array metadata on the given array instance by deserializing the
@@ -336,7 +377,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_array_metadata(
     tiledb_ctx_t* ctx,
     tiledb_array_t* array,
     tiledb_serialization_type_t serialization_type,
-    const tiledb_buffer_t* buffer);
+    const tiledb_buffer_t* buffer) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the given query's estimated result sizes map.
@@ -356,7 +397,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_query_est_result_sizes(
     const tiledb_query_t* query,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes into an existing query from the given buffer.
@@ -374,7 +415,7 @@ TILEDB_EXPORT int32_t tiledb_deserialize_query_est_result_sizes(
     tiledb_query_t* query,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    const tiledb_buffer_t* buffer);
+    const tiledb_buffer_t* buffer) TILEDB_NOEXCEPT;
 
 /**
  * Serializes the given config.
@@ -395,7 +436,7 @@ TILEDB_EXPORT int32_t tiledb_serialize_config(
     const tiledb_config_t* config,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_buffer_t** buffer);
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
 
 /**
  * Deserializes a new config from the given buffer.
@@ -413,8 +454,87 @@ TILEDB_EXPORT int32_t tiledb_deserialize_config(
     const tiledb_buffer_t* buffer,
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
-    tiledb_config_t** config);
+    tiledb_config_t** config) TILEDB_NOEXCEPT;
 
+/**
+ * Serializes the given group.
+ *
+ * Where possible the serialization is zero-copy. The returned buffer list
+ * contains an ordered list of pointers to buffers that logically contain the
+ * entire serialized group when concatenated.
+ *
+ * @note The caller must free the returned `tiledb_buffer_list_t`.
+ *
+ * @param ctx The TileDB context.
+ * @param group The group.
+ * @param serialization_type Type of serialization to use
+ * @param client_side If set to 1, deserialize from "client-side" perspective.
+ *    Else, "server-side."
+ * @param tiledb_buffer_t Will be set to a newly allocated buffer containing
+ *    the serialized group.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_serialize_group(
+    tiledb_ctx_t* ctx,
+    const tiledb_group_t* group,
+    tiledb_serialization_type_t serialize_type,
+    int32_t client_side,
+    tiledb_buffer_t** buffer_list) TILEDB_NOEXCEPT;
+
+/**
+ * Deserializes into an existing group from the given buffer.
+ *
+ * @note The deserialization is zero-copy, so the source buffer must exceed
+ * the lifetime of the group being deserialized to.
+ *
+ * @param ctx The TileDB context.
+ * @param buffer Buffer to deserialize from
+ * @param serialization_type Type of deserialization to use
+ * @param client_side If set to 1, deserialize from "client-side" perspective.
+ *    Else, "server-side."
+ * @param group The group object to deserialize into (must be pre-allocated).
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_deserialize_group(
+    tiledb_ctx_t* ctx,
+    const tiledb_buffer_t* buffer,
+    tiledb_serialization_type_t serialize_type,
+    int32_t client_side,
+    tiledb_group_t* group) TILEDB_NOEXCEPT;
+
+/**
+ * Serializes the group metadata into the given buffer.
+ *
+ * @note The caller must free the returned `tiledb_buffer_t`.
+ *
+ * @param ctx The TileDB context.
+ * @param group Group whose metadata to serialize.
+ * @param serialization_type Type of serialization to use
+ * @param buffer Will be set to a newly allocated buffer containing the
+ *      serialized group metadata.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_serialize_group_metadata(
+    tiledb_ctx_t* ctx,
+    const tiledb_group_t* group,
+    tiledb_serialization_type_t serialization_type,
+    tiledb_buffer_t** buffer) TILEDB_NOEXCEPT;
+
+/**
+ * Sets the group metadata on the given group instance by deserializing the
+ * group metadata from the given buffer.
+ *
+ * @param ctx The TileDB context.
+ * @param group Group whose metadata to set.
+ * @param serialization_type Type of serialization to use
+ * @param buffer Buffer containing serialized group metadata.
+ * @return `TILEDB_OK` for success and `TILEDB_ERR` for error.
+ */
+TILEDB_EXPORT int32_t tiledb_deserialize_group_metadata(
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    tiledb_serialization_type_t serialization_type,
+    const tiledb_buffer_t* buffer) TILEDB_NOEXCEPT;
 #ifdef __cplusplus
 }
 #endif
