@@ -293,16 +293,20 @@ Status FragmentConsolidator::vacuum(const char* array_name) {
         URI(array_name),
         0,
         std::numeric_limits<uint64_t>::max(),
-        config_.with_timestamps_,
         ArrayDirectoryMode::VACUUM_FRAGMENTS);
   } catch (const std::logic_error& le) {
     return LOG_STATUS(Status_ArrayDirectoryError(le.what()));
   }
 
-  const auto& fragment_uris_to_vacuum = array_dir.fragment_uris_to_vacuum();
-  const auto& commit_uris_to_vacuum = array_dir.commit_uris_to_vacuum();
-  const auto& commit_uris_to_ignore = array_dir.commit_uris_to_ignore();
-  const auto& vac_uris_to_vacuum = array_dir.fragment_vac_uris_to_vacuum();
+  auto filtered_fragment_uris = array_dir.filtered_fragment_uris(true);
+  const auto& fragment_uris_to_vacuum =
+      filtered_fragment_uris.fragment_uris_to_vacuum();
+  const auto& commit_uris_to_vacuum =
+      filtered_fragment_uris.commit_uris_to_vacuum();
+  const auto& commit_uris_to_ignore =
+      filtered_fragment_uris.commit_uris_to_ignore();
+  const auto& vac_uris_to_vacuum =
+      filtered_fragment_uris.fragment_vac_uris_to_vacuum();
 
   if (commit_uris_to_ignore.size() > 0) {
     // Write an ignore file to ensure consolidated WRT files still work
@@ -816,6 +820,12 @@ Status FragmentConsolidator::set_config(const Config* config) {
   RETURN_NOT_OK(merged_config.get<float>(
       "sm.consolidation.step_size_ratio", &config_.size_ratio_, &found));
   assert(found);
+  config_.purge_deleted_cells_ = false;
+  RETURN_NOT_OK(merged_config.get<bool>(
+      "sm.consolidation.purge_deleted_cells",
+      &config_.purge_deleted_cells_,
+      &found));
+  assert(found);
   config_.min_frags_ = 0;
   RETURN_NOT_OK(merged_config.get<uint32_t>(
       "sm.consolidation.step_min_frags", &config_.min_frags_, &found));
@@ -830,14 +840,11 @@ Status FragmentConsolidator::set_config(const Config* config) {
   RETURN_NOT_OK(merged_config.get<uint64_t>(
       "sm.consolidation.timestamp_end", &config_.timestamp_end_, &found));
   assert(found);
-  config_.with_timestamps_ = false;
-  RETURN_NOT_OK(merged_config.get<bool>(
-      "sm.consolidation.with_timestamps", &config_.with_timestamps_, &found));
-  assert(found);
   std::string reader =
       merged_config.get("sm.query.sparse_global_order.reader", &found);
   assert(found);
   config_.use_refactored_reader_ = reader.compare("refactored") == 0;
+  config_.with_timestamps_ = true;
 
   // Sanity checks
   if (config_.min_frags_ > config_.max_frags_)
@@ -852,10 +859,6 @@ Status FragmentConsolidator::set_config(const Config* config) {
     return logger_->status(
         Status_ConsolidatorError("Invalid configuration; Amplification config "
                                  "parameter must be non-negative"));
-  if (config_.with_timestamps_ && !config_.use_refactored_reader_)
-    return logger_->status(
-        Status_ConsolidatorError("Invalid configuration; Consolidation with "
-                                 "timestamps requires refactored reader"));
 
   return Status::Ok();
 }
