@@ -311,18 +311,6 @@ class Config {
    *    `fragment_meta` (remove only consolidated fragment metadata), or
    *    `array_meta` (remove consolidated array metadata files). <br>
    *    **Default**: fragments
-   * - `sm.vacuum.timestamp_start` <br>
-   *    **Experimental** <br>
-   *    When set, an array will be vacuumed between this value and
-   *    `sm.vacuum.timestamp_end` (inclusive). <br>
-   *    Only for `fragments` and `array_meta` vacuum mode. <br>
-   *    **Default**: 0
-   * - `sm.vacuum.timestamp_end` <br>
-   *    **Experimental** <br>
-   *    When set, an array will be vacuumed between `sm.vacuum.timestamp_start`
-   *    and this value (inclusive). <br>
-   *    Only for `fragments` and `array_meta` vacuum mode. <br>
-   *    **Default**: UINT64_MAX
    * - `sm.consolidation_mode` <br>
    *    The consolidation mode, one of `fragments` (consolidate all fragments),
    *    `fragment_meta` (consolidate only fragment metadata footers to a single
@@ -345,6 +333,10 @@ class Config {
    *    The number of consolidation steps to be performed when executing
    *    the consolidation algorithm.<br>
    *    **Default**: 1
+   * - `sm.consolidation.purge_deleted_cells` <br>
+   *    **Experimental** <br>
+   *    Purge deleted cells from the consolidated fragment or not.<br>
+   *    **Default**: false
    * - `sm.consolidation.step_min_frags` <br>
    *    The minimum number of fragments to consolidate in a single step.<br>
    *    **Default**: UINT32_MAX
@@ -389,7 +381,7 @@ class Config {
    *    **Default**: bytes
    * - `sm.query.dense.reader` <br>
    *    Which reader to use for dense queries. "refactored" or "legacy".<br>
-   *    **Default**: lagacy
+   *    **Default**: refactored
    * - `sm.query.sparse_global_order.reader` <br>
    *    Which reader to use for sparse global order queries. "refactored"
    *    or "legacy".<br>
@@ -421,10 +413,6 @@ class Config {
    *    Ratio of the budget allocated for array data in the sparse global
    *    order reader. <br>
    *    **Default**: 0.1
-   * - `sm.mem.reader.sparse_global_order.ratio_rcs` <br>
-   *    Ratio of the budget allocated for result cell slabs in the sparse
-   *    global order reader. <br>
-   *    **Default**: 0.05
    * - `sm.mem.reader.sparse_unordered_with_dups.ratio_coords` <br>
    *    Ratio of the budget allocated for coordinates in the sparse unordered
    *    with duplicates reader. <br>
@@ -443,6 +431,13 @@ class Config {
    *    **Default**: 0.1
    *    The maximum byte size to read-ahead from the backend. <br>
    *    **Default**: 102400
+   * - `sm.group.timestamp_start` <br>
+   *    The start timestamp used for opening the group. <br>
+   *    **Default**: 0
+   * - `sm.group.timestamp_end` <br>
+   *    The end timestamp used for opening the group. <br>
+   *    Also used for the write timestamp if set. <br>
+   *    **Default**: UINT64_MAX
    * -  `vfs.read_ahead_cache_size` <br>
    *    The the total maximum size of the read-ahead cache, which is an LRU.
    *    <br>
@@ -452,6 +447,9 @@ class Config {
    *    (except parallel S3 writes, which are controlled by
    *    `vfs.s3.multipart_part_size`.) <br>
    *    **Default**: 10MB
+   * - `vfs.max_batch_size` <br>
+   *    The maximum number of bytes in a VFS read operation<br>
+   *    **Default**: UINT64_MAX
    * - `vfs.min_batch_size` <br>
    *    The minimum number of bytes in a VFS read operation<br>
    *    **Default**: 20MB
@@ -468,10 +466,6 @@ class Config {
    *    The maximum number of parallel operations on objects with `file:///`
    *    URIs. <br>
    *    **Default**: `sm.io_concurrency_level`
-   * - `vfs.file.enable_filelocks` <br>
-   *    If set to `false`, file locking operations are no-ops for `file:///`
-   *    URIs in VFS. <br>
-   *    **Default**: `true`
    * - `vfs.azure.storage_account_name` <br>
    *    Set the Azure Storage Account name. <br>
    *    **Default**: ""
@@ -713,6 +707,27 @@ class Config {
    *    The delay factor to exponentially wait until further retries of a
    *    failed REST request <br>
    *    **Default**: 1.25
+   * - `rest.curl.verbose` <br>
+   *    Set curl to run in verbose mode for REST requests <br>
+   *    curl will print to stdout with this option
+   *    **Default**: false
+   * - `rest.load_metadata_on_array_open` <br>
+   *    If true, array metadata will be loaded and sent to server together with
+   *    the open array <br>
+   *    **Default**: true
+   * - `rest.load_non_empty_domain_on_array_open` <br>
+   *    If true, array non empty domain will be loaded and sent to server
+   *    together with the open array <br>
+   *    **Default**: true
+   * - `rest.use_refactored_array_open` <br>
+   *    If true, the new, experimental REST routes and APIs for opening an array
+   *    will be used <br>
+   *    **Default**: false
+   * - `filestore.buffer_size` <br>
+   *    Specifies the size in bytes of the internal buffers used in the
+   *    filestore API. The size should be bigger than the minimum tile size
+   *    filestore currently supports, that is currently 1024bytes. <br>
+   *    **Default**: 100MB
    */
   Config& set(const std::string& param, const std::string& value) {
     tiledb_error_t* err;
@@ -739,6 +754,19 @@ class Config {
     return val;
   }
 
+  /**
+   * Check if a configuration parameter exists.
+   * @param param Name of configuration parameter
+   * @return true if the parameter exists, false otherwise
+   */
+  bool contains(const std::string_view& param) const {
+    const char* val;
+    tiledb_error_t* err;
+    tiledb_config_get(config_.get(), param.data(), &val, &err);
+
+    return val != nullptr;
+  }
+
   /**
    * Operator that enables setting parameters with `[]`.
    *
