@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -275,6 +275,10 @@ class Filter {
         return "CHECKSUM_MD5";
       case TILEDB_FILTER_CHECKSUM_SHA256:
         return "CHECKSUM_SHA256";
+      case TILEDB_FILTER_DICTIONARY:
+        return "DICTIONARY_ENCODING";
+      case TILEDB_FILTER_SCALE_FLOAT:
+        return "SCALE_FLOAT";
     }
     return "";
   }
@@ -313,6 +317,15 @@ class Filter {
         if (!std::is_same<uint32_t, T>::value)
           throw std::invalid_argument("Option value must be uint32_t.");
         break;
+      case TILEDB_SCALE_FLOAT_BYTEWIDTH:
+        if (!std::is_same<uint64_t, T>::value)
+          throw std::invalid_argument("Option value must be uint64_t.");
+        break;
+      case TILEDB_SCALE_FLOAT_FACTOR:
+      case TILEDB_SCALE_FLOAT_OFFSET:
+        if (!std::is_same<double, T>::value)
+          throw std::invalid_argument("Option value must be double.");
+        break;
       default:
         throw std::invalid_argument("Invalid option type");
     }
