@@ -69,11 +69,18 @@ inline Status array_type_enum(
   else if (array_type_str == constants::sparse_str)
     *array_type_enum = ArrayType::SPARSE;
   else {
-    return Status::Error("Invalid ArrayType " + array_type_str);
+    return Status_Error("Invalid ArrayType " + array_type_str);
   }
   return Status::Ok();
 }
 
+inline Status array_type_is_valid(uint8_t array_type_enum) {
+  if (array_type_enum != 0 && array_type_enum != 1)
+    return Status_Error("Invalid ArrayType enum.");
+
+  return Status::Ok();
+}
+
 }  // namespace sm
 }  // namespace tiledb
 
