@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -59,7 +59,9 @@ enum class Compressor : uint8_t {
   /** Bzip2 compressor */
   BZIP2 = 5,
   /** Double-delta compressor */
-  DOUBLE_DELTA = 6
+  DOUBLE_DELTA = 6,
+  /** Dictionary compressor */
+  DICTIONARY_ENCODING = 7,
 };
 
 /** Returns the string representation of the input compressor. */
@@ -79,6 +81,8 @@ inline const std::string& compressor_str(Compressor type) {
       return constants::bzip2_str;
     case Compressor::DOUBLE_DELTA:
       return constants::double_delta_str;
+    case Compressor::DICTIONARY_ENCODING:
+      return constants::filter_dictionary_str;
     default:
       return constants::empty_str;
   }
@@ -101,8 +105,10 @@ inline Status compressor_enum(
     *compressor = Compressor::BZIP2;
   else if (compressor_type_str == constants::double_delta_str)
     *compressor = Compressor::DOUBLE_DELTA;
+  else if (compressor_type_str == constants::filter_dictionary_str)
+    *compressor = Compressor::DICTIONARY_ENCODING;
   else {
-    return Status::Error("Invalid Compressor " + compressor_type_str);
+    return Status_Error("Invalid Compressor " + compressor_type_str);
   }
   return Status::Ok();
 }
