@@ -34,6 +34,11 @@
 #ifndef TILEDB_OBJECT_TYPE_H
 #define TILEDB_OBJECT_TYPE_H
 
+#include "tiledb/common/status.h"
+#include "tiledb/sm/misc/constants.h"
+
+using namespace tiledb::common;
+
 namespace tiledb {
 namespace sm {
 
@@ -67,7 +72,7 @@ inline Status object_type_enum(
   else if (object_type_str == constants::object_type_array_str)
     *object_type = ObjectType::ARRAY;
   else
-    return Status::Error("Invalid ObjectType " + object_type_str);
+    return Status_Error("Invalid ObjectType " + object_type_str);
 
   return Status::Ok();
 }
