@@ -43,11 +43,16 @@
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/misc/constants.h"
-#include "tiledb/sm/misc/uri.h"
+#include "uri.h"
 
 using namespace tiledb::common;
 
 namespace tiledb {
+
+namespace common::filesystem {
+class directory_entry;
+}
+
 namespace sm {
 
 class GCS {
@@ -205,6 +210,21 @@ class GCS {
       const std::string& delimiter = "/",
       int max_paths = -1) const;
 
+  /**
+   *
+   * Lists objects and object information that start with `prefix`.
+   *
+   * @param uri The parent path to list sub-paths.
+   * @param delimiter The uri is truncated to the first delimiter
+   * @param max_paths The maximum number of paths to be retrieved
+   * @return A list of directory_entry objects
+   */
+  tuple<Status, optional<std::vector<filesystem::directory_entry>>>
+  ls_with_sizes(
+      const URI& uri,
+      const std::string& delimiter = "/",
+      int max_paths = -1) const;
+
   /**
    * Renames an object.
    *
