@@ -39,17 +39,17 @@
 #include <string>
 #include <vector>
 
+#include "tiledb/common/common.h"
+#include "tiledb/common/filesystem/directory_entry.h"
 #include "tiledb/common/macros.h"
 #include "tiledb/common/status.h"
-#include "tiledb/common/thread_pool.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/cache/lru_cache.h"
 #include "tiledb/sm/config/config.h"
-#include "tiledb/sm/filesystem/filelock.h"
 #include "tiledb/sm/filesystem/mem_filesystem.h"
 #include "tiledb/sm/misc/cancelable_tasks.h"
-#include "tiledb/sm/misc/uri.h"
 #include "tiledb/sm/stats/stats.h"
+#include "uri.h"
 
 #ifdef _WIN32
 #include "tiledb/sm/filesystem/win.h"
@@ -199,27 +199,6 @@ class VFS {
    */
   Status remove_file(const URI& uri) const;
 
-  /**
-   * Locks a filelock.
-   *
-   * @param uri The URI of the filelock.
-   * @param lock A handle for the filelock (used in unlocking the
-   *     filelock).
-   * @param shared *True* if it is a shared lock, *false* if it is an
-   *     exclusive lock.
-   * @return Status
-   */
-  Status filelock_lock(const URI& uri, filelock_t* lock, bool shared) const;
-
-  /**
-   * Unlocks a filelock.
-   *
-   * @param uri The URI of the filelock.
-   * @param lock The handle of the filelock.
-   * @return Status
-   */
-  Status filelock_unlock(const URI& uri) const;
-
   /**
    * Retrieves the size of a file.
    *
@@ -300,6 +279,15 @@ class VFS {
    */
   Status ls(const URI& parent, std::vector<URI>* uris) const;
 
+  /**
+   * Retrieves all the entries contained in the parent.
+   *
+   * @param parent The target directory to list.
+   * @return All entries that are contained in the parent
+   */
+  tuple<Status, optional<std::vector<filesystem::directory_entry>>>
+  ls_with_sizes(const URI& parent) const;
+
   /**
    * Renames a file.
    *
@@ -366,7 +354,7 @@ class VFS {
    */
   Status read_all(
       const URI& uri,
-      const std::vector<std::tuple<uint64_t, Tile*, uint64_t>>& regions,
+      const std::vector<tuple<uint64_t, Tile*, uint64_t>>& regions,
       ThreadPool* thread_pool,
       std::vector<ThreadPool::Task>* tasks,
       bool use_read_ahead = true);
@@ -434,7 +422,7 @@ class VFS {
    */
   struct BatchedRead {
     /** Construct a BatchedRead consisting of the single given region. */
-    BatchedRead(const std::tuple<uint64_t, Tile*, uint64_t>& region) {
+    BatchedRead(const tuple<uint64_t, Tile*, uint64_t>& region) {
       offset = std::get<0>(region);
       nbytes = std::get<2>(region);
       regions.push_back(region);
@@ -450,7 +438,7 @@ class VFS {
      * Original regions making up the batch. Vector of tuples of the form
      * (offset, dest_buffer, nbytes).
      */
-    std::vector<std::tuple<uint64_t, Tile*, uint64_t>> regions;
+    std::vector<tuple<uint64_t, Tile*, uint64_t>> regions;
   };
 
   /**
@@ -676,7 +664,7 @@ class VFS {
    * @return Status
    */
   Status compute_read_batches(
-      const std::vector<std::tuple<uint64_t, Tile*, uint64_t>>& regions,
+      const std::vector<tuple<uint64_t, Tile*, uint64_t>>& regions,
       std::vector<BatchedRead>* batches) const;
 
   /**
@@ -716,15 +704,6 @@ class VFS {
       const uint64_t nbytes,
       const bool use_read_ahead);
 
-  /**
-   * Decrement the lock count of the given URI.
-   *
-   * @param uri The URI
-   * @param lock The filelock_t that is held, or INVALID_FILELOCK
-   * @return Status
-   */
-  Status decr_lock_count(const URI& uri, bool* is_zero, filelock_t* lock) const;
-
   /**
    * Retrieves the backend-specific max number of parallel operations for VFS
    * read.
