@@ -33,9 +33,11 @@
 
 #include "win.h"
 #include "path_win.h"
+#include "tiledb/common/common.h"
 #include "tiledb/common/filesystem/directory_entry.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
+#include "tiledb/common/scoped_executor.h"
 #include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/misc/tdb_math.h"
@@ -47,12 +49,14 @@
 #endif
 #include <Shlwapi.h>
 #include <Windows.h>
+#include <strsafe.h>
 #include <wininet.h>  // For INTERNET_MAX_URL_LENGTH
 #include <algorithm>
 #include <cassert>
 #include <fstream>
 #include <iostream>
 #include <sstream>
+#include <string_view>
 
 using namespace tiledb::common;
 using tiledb::common::filesystem::directory_entry;
@@ -62,25 +66,47 @@ namespace sm {
 
 namespace {
 /** Returns the last Windows error message string. */
-std::string get_last_error_msg() {
-  DWORD err = GetLastError();
-  LPVOID lpMsgBuf;
+std::string get_last_error_msg_desc(decltype(GetLastError()) gle) {
+  LPVOID lpMsgBuf = nullptr;
   if (FormatMessage(
           FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |
               FORMAT_MESSAGE_IGNORE_INSERTS,
           NULL,
-          err,
+          gle,
           MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
           (LPTSTR)&lpMsgBuf,
           0,
           NULL) == 0) {
-    LocalFree(lpMsgBuf);
+    if (lpMsgBuf) {
+      LocalFree(lpMsgBuf);
+    }
     return "unknown error";
   }
   std::string msg(reinterpret_cast<char*>(lpMsgBuf));
   LocalFree(lpMsgBuf);
   return msg;
 }
+
+std::string get_last_error_msg(
+    decltype(GetLastError()) gle, const std::string_view func_desc) {
+  std::string gle_desc = get_last_error_msg_desc(gle);
+
+  auto buf_len = gle_desc.length() + func_desc.size() + 60;
+  auto display_buf = std::make_unique<char[]>(buf_len);
+  std::snprintf(
+      display_buf.get(),
+      buf_len,
+      "%s GetLastError %lu (0x%08lx): %s",
+      func_desc.data(),
+      gle,
+      gle,
+      gle_desc.c_str());
+  return {display_buf.get()};
+}
+std::string get_last_error_msg(const std::string_view func_desc) {
+  DWORD gle = GetLastError();
+  return get_last_error_msg(gle, func_desc);
+}
 }  // namespace
 
 std::string Win::abs_path(const std::string& path) {
@@ -99,7 +125,10 @@ std::string Win::abs_path(const std::string& path) {
   char result[MAX_PATH];
   std::string str_result;
   if (PathCanonicalize(result, full_path.c_str()) == FALSE) {
-    LOG_STATUS(Status_IOError(std::string("Cannot canonicalize path.")));
+    auto gle = GetLastError();
+    LOG_STATUS(Status_IOError(std::string(
+        "Cannot canonicalize path. (" +
+        get_last_error_msg(gle, "PathCanonicalize") + ")")));
   } else {
     str_result = result;
   }
@@ -115,7 +144,7 @@ Status Win::create_dir(const std::string& path) const {
   if (CreateDirectory(path.c_str(), nullptr) == 0) {
     return LOG_STATUS(Status_IOError(
         std::string("Cannot create directory '") + path +
-        "': " + get_last_error_msg()));
+        "': " + get_last_error_msg("CreateDirectory")));
   }
   return Status::Ok();
 }
@@ -133,9 +162,16 @@ Status Win::touch(const std::string& filename) const {
       CREATE_NEW,
       FILE_ATTRIBUTE_NORMAL,
       nullptr);
-  if (file_h == INVALID_HANDLE_VALUE || CloseHandle(file_h) == 0) {
+  auto closefileonexit = [&]() {
+    if (file_h != INVALID_HANDLE_VALUE)
+      CloseHandle(file_h);
+  };
+  tiledb::common::ScopedExecutor onexit1(closefileonexit);
+  if (file_h == INVALID_HANDLE_VALUE) {
+    auto gle = GetLastError();
     return LOG_STATUS(Status_IOError(
-        std::string("Failed to create file '") + filename + "'"));
+        std::string("Failed to create file '") + filename + " (" +
+        get_last_error_msg(gle, "CreateFile") + ")'"));
   }
   return Status::Ok();
 }
@@ -145,7 +181,9 @@ std::string Win::current_dir() {
   unsigned long length = GetCurrentDirectory(0, nullptr);
   char* path = (char*)tdb_malloc(length * sizeof(char));
   if (path == nullptr || GetCurrentDirectory(length, path) == 0) {
-    LOG_STATUS(Status_IOError(std::string("Failed to get current directory.")));
+    LOG_STATUS(Status_IOError(std::string(
+        "Failed to get current directory. " +
+        get_last_error_msg("GetCurrentDirectory"))));
   }
   dir = path;
   tdb_free(path);
@@ -155,6 +193,7 @@ std::string Win::current_dir() {
 Status Win::recursively_remove_directory(const std::string& path) const {
   const std::string glob = path + "\\*";
   WIN32_FIND_DATA find_data;
+  const char* what_call = nullptr;
 
   // Get first file in directory.
   HANDLE find_h = FindFirstFileEx(
@@ -165,6 +204,7 @@ Status Win::recursively_remove_directory(const std::string& path) const {
       nullptr,
       0);
   if (find_h == INVALID_HANDLE_VALUE) {
+    what_call = "FindFirstFileEx";
     goto err;
   }
 
@@ -175,10 +215,12 @@ Status Win::recursively_remove_directory(const std::string& path) const {
       std::string file_path = path + "\\" + find_data.cFileName;
       if (PathIsDirectory(file_path.c_str())) {
         if (!recursively_remove_directory(file_path).ok()) {
+          what_call = "recursively_remove_directory";
           goto err;
         }
       } else {
         if (!remove_file(file_path).ok()) {
+          what_call = "remove_file";
           goto err;
         }
       }
@@ -191,6 +233,7 @@ Status Win::recursively_remove_directory(const std::string& path) const {
   }
 
   if (RemoveDirectory(path.c_str()) == 0) {
+    what_call = "RemoveDirectory";
     goto err;
   }
 
@@ -198,11 +241,18 @@ Status Win::recursively_remove_directory(const std::string& path) const {
   return Status::Ok();
 
 err:
+  auto gle = GetLastError();
   if (find_h != INVALID_HANDLE_VALUE) {
     FindClose(find_h);
   }
-  return LOG_STATUS(
-      Status_IOError(std::string("Failed to remove directory '" + path + "'")));
+  std::string offender = __func__;
+  if (what_call) {
+    offender.append(" ");
+    offender.append(what_call);
+  }
+  return LOG_STATUS(Status_IOError(std::string(
+      "Failed to remove directory '" + path + "' " +
+      get_last_error_msg(gle, offender.c_str()))));
 }
 
 Status Win::remove_dir(const std::string& path) const {
@@ -216,8 +266,9 @@ Status Win::remove_dir(const std::string& path) const {
 
 Status Win::remove_file(const std::string& path) const {
   if (!DeleteFile(path.c_str())) {
-    return LOG_STATUS(
-        Status_IOError(std::string("Failed to delete file '" + path + "'")));
+    return LOG_STATUS(Status_IOError(std::string(
+        "Failed to delete file '" + path + "' " +
+        get_last_error_msg("DeleteFile"))));
   }
   return Status::Ok();
 }
@@ -233,13 +284,15 @@ Status Win::file_size(const std::string& path, uint64_t* size) const {
       FILE_ATTRIBUTE_NORMAL,
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
-    return LOG_STATUS(Status_IOError(
-        std::string("Failed to get file size for '" + path + "'")));
+    return LOG_STATUS(Status_IOError(std::string(
+        "Failed to get file size for '" + path + "' (" +
+        get_last_error_msg("CreateFile") + ")'")));
   }
   if (!GetFileSizeEx(file_h, &nbytes)) {
     CloseHandle(file_h);
-    return LOG_STATUS(Status_IOError(
-        std::string("Failed to get file size for '" + path + "'")));
+    return LOG_STATUS(Status_IOError(std::string(
+        "Failed to get file size for '" + path + "' (" +
+        get_last_error_msg("GetFileSizeEx") + ")'")));
   }
   *size = nbytes.QuadPart;
   CloseHandle(file_h);
@@ -284,6 +337,7 @@ tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
   const std::string glob = path + (ends_with_slash ? "*" : "\\*");
   WIN32_FIND_DATA find_data;
   std::vector<directory_entry> entries;
+  const char* what_call = nullptr;
 
   // Get first file in directory.
   HANDLE find_h = FindFirstFileEx(
@@ -294,6 +348,7 @@ tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
       nullptr,
       0);
   if (find_h == INVALID_HANDLE_VALUE) {
+    what_call = "FindFirstFileEx";
     goto err;
   }
 
@@ -304,12 +359,12 @@ tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
       std::string file_path =
           path + (ends_with_slash ? "" : "\\") + find_data.cFileName;
       if (is_dir(file_path)) {
-        entries.emplace_back(file_path, 0);
+        entries.emplace_back(file_path, 0, true);
       } else {
         ULARGE_INTEGER size;
         size.LowPart = find_data.nFileSizeLow;
         size.HighPart = find_data.nFileSizeHigh;
-        entries.emplace_back(file_path, size.QuadPart);
+        entries.emplace_back(file_path, size.QuadPart, false);
       }
     }
 
@@ -323,10 +378,18 @@ tuple<Status, optional<std::vector<directory_entry>>> Win::ls_with_sizes(
   return {Status::Ok(), entries};
 
 err:
+  auto gle = GetLastError();
   if (find_h != INVALID_HANDLE_VALUE) {
     FindClose(find_h);
   }
-  std::string errmsg("Failed to list directory \"" + path + "\"");
+  std::string offender = __func__;
+  if (what_call) {
+    offender.append(" ");
+    offender.append(what_call);
+  }
+  std::string errmsg(
+      "Failed to list directory \"" + path + "\" " +
+      get_last_error_msg(gle, offender.c_str()));
   auto st = LOG_STATUS(Status_IOError(errmsg));
   return {st, nullopt};
 }
@@ -336,7 +399,8 @@ Status Win::move_path(
   if (MoveFileEx(
           old_path.c_str(), new_path.c_str(), MOVEFILE_REPLACE_EXISTING) == 0) {
     return LOG_STATUS(Status_IOError(std::string(
-        "Failed to rename '" + old_path + "' to '" + new_path + "'.")));
+        "Failed to rename '" + old_path + "' to '" + new_path + "'. (" +
+        get_last_error_msg("MoveFileEx") + ")")));
   }
   return Status::Ok();
 }
@@ -358,28 +422,36 @@ Status Win::read(
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
     return LOG_STATUS(Status_IOError(
-        "Cannot read from file '" + path + "'; File opening error"));
+        "Cannot read from file '" + path + "'; File opening error (" +
+        get_last_error_msg("CreateFile") + ")"));
   }
 
   LARGE_INTEGER offset_lg_int;
   offset_lg_int.QuadPart = offset;
   if (SetFilePointerEx(file_h, offset_lg_int, NULL, FILE_BEGIN) == 0) {
+    auto gle = GetLastError();
     CloseHandle(file_h);
     return LOG_STATUS(Status_IOError(
-        "Cannot read from file '" + path + "'; File seek error"));
+        "Cannot read from file '" + path + "'; File seek error " +
+        get_last_error_msg(gle, "SetFilePointerEx")));
   }
 
   unsigned long num_bytes_read = 0;
   if (ReadFile(file_h, buffer, nbytes, &num_bytes_read, NULL) == 0 ||
       num_bytes_read != nbytes) {
+    auto gle = GetLastError();
     CloseHandle(file_h);
     return LOG_STATUS(Status_IOError(
-        "Cannot read from file '" + path + "'; File read error"));
+        "Cannot read from file '" + path + "'; File read error " +
+        (gle != 0 ? get_last_error_msg(gle, "ReadFile") :
+                    "num_bytes_read " + std::to_string(num_bytes_read) +
+                        " != nbyes " + std::to_string(nbytes))));
   }
 
   if (CloseHandle(file_h) == 0) {
     return LOG_STATUS(Status_IOError(
-        "Cannot read from file '" + path + "'; File closing error"));
+        "Cannot read from file '" + path + "'; File closing error " +
+        get_last_error_msg("CloseHandle")));
   }
 
   return Status::Ok();
@@ -401,19 +473,23 @@ Status Win::sync(const std::string& path) const {
       FILE_ATTRIBUTE_NORMAL,
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
-    return LOG_STATUS(
-        Status_IOError("Cannot sync file '" + path + "'; File opening error"));
+    return LOG_STATUS(Status_IOError(
+        "Cannot sync file '" + path + "'; File opening error " +
+        get_last_error_msg("CreateFile")));
   }
 
   if (FlushFileBuffers(file_h) == 0) {
+    auto gle = GetLastError();
     CloseHandle(file_h);
-    return LOG_STATUS(
-        Status_IOError("Cannot sync file '" + path + "'; Sync error"));
+    return LOG_STATUS(Status_IOError(
+        "Cannot sync file '" + path + "'; Sync error " +
+        get_last_error_msg(gle, "FlushFileBuffers")));
   }
 
   if (CloseHandle(file_h) == 0) {
     return LOG_STATUS(Status_IOError(
-        "Cannot read from file '" + path + "'; File closing error"));
+        "Cannot read from file '" + path + "'; File closing error " +
+        get_last_error_msg("CloseHandle")));
   }
 
   return Status::Ok();
@@ -444,14 +520,17 @@ Status Win::write(
       NULL);
   if (file_h == INVALID_HANDLE_VALUE) {
     return LOG_STATUS(Status_IOError(
-        "Cannot write to file '" + path + "'; File opening error"));
+        "Cannot write to file '" + path + "'; File opening error " +
+        get_last_error_msg("CreateFile")));
   }
   // Get the current file size.
   LARGE_INTEGER file_size_lg_int;
   if (!GetFileSizeEx(file_h, &file_size_lg_int)) {
+    auto gle = GetLastError();
     CloseHandle(file_h);
-    return LOG_STATUS(
-        Status_IOError("Cannot write to file '" + path + "'; File size error"));
+    return LOG_STATUS(Status_IOError(
+        "Cannot write to file '" + path + "'; File size error " +
+        get_last_error_msg(gle, "GetFileSizeEx")));
   }
   uint64_t file_offset = file_size_lg_int.QuadPart;
   // Ensure that each thread is responsible for at least min_parallel_size
@@ -484,6 +563,7 @@ Status Win::write(
     if (!st.ok()) {
       CloseHandle(file_h);
       std::stringstream errmsg;
+      // failures in write_at() log their own gle messages.
       errmsg << "Cannot write to file '" << path << "'; " << st.message();
       return LOG_STATUS(Status_IOError(errmsg.str()));
     }
@@ -524,7 +604,7 @@ Status Win::write_at(
         bytes_written != constants::max_write_bytes) {
       return LOG_STATUS(Status_IOError(std::string(
           "Cannot write to file; File writing error: " +
-          get_last_error_msg())));
+          get_last_error_msg("WriteFile"))));
     }
     buffer_size -= constants::max_write_bytes;
     byte_idx += constants::max_write_bytes;
@@ -540,7 +620,8 @@ Status Win::write_at(
           0 ||
       bytes_written != buffer_size) {
     return LOG_STATUS(Status_IOError(std::string(
-        "Cannot write to file; File writing error: " + get_last_error_msg())));
+        "Cannot write to file; File writing error: " +
+        get_last_error_msg("WriteFile"))));
   }
   return Status::Ok();
 }
