@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -78,6 +78,8 @@ class ByteshuffleFilter : public Filter {
    * Shuffle the bytes of the input data into the output data buffer.
    */
   Status run_forward(
+      const Tile& tile,
+      Tile* const tile_offsets,
       FilterBuffer* input_metadata,
       FilterBuffer* input,
       FilterBuffer* output_metadata,
@@ -87,6 +89,8 @@ class ByteshuffleFilter : public Filter {
    * Unshuffle the bytes of the input data into the output data buffer.
    */
   Status run_reverse(
+      const Tile& tile,
+      Tile* const tile_offsets,
       FilterBuffer* input_metadata,
       FilterBuffer* input,
       FilterBuffer* output_metadata,
@@ -100,20 +104,24 @@ class ByteshuffleFilter : public Filter {
   /**
    * Perform byte shuffling on the given input buffer.
    *
+   * @param tile Current tile on which the filter is being run
    * @param part Buffer containing data to be shuffled.
    * @param output Buffer to hold shuffled data.
    * @return Status
    */
-  Status shuffle_part(const ConstBuffer* part, Buffer* output) const;
+  Status shuffle_part(
+      const Tile& tile, const ConstBuffer* part, Buffer* output) const;
 
   /**
    * Perform byte unshuffling on the given input buffer.
    *
+   * @param tile Current tile on which the filter is being run
    * @param part Buffer containing shuffled data.
    * @param output Buffer to hold unshuffled data.
    * @return Status
    */
-  Status unshuffle_part(const ConstBuffer* input, Buffer* output) const;
+  Status unshuffle_part(
+      const Tile& tile, const ConstBuffer* part, Buffer* output) const;
 };
 
 }  // namespace sm
