@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2022 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -61,6 +61,8 @@ void ChecksumSHA256Filter::dump(FILE* out) const {
 }
 
 Status ChecksumSHA256Filter::run_forward(
+    const Tile&,
+    Tile* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -94,6 +96,8 @@ Status ChecksumSHA256Filter::run_forward(
 }
 
 Status ChecksumSHA256Filter::run_reverse(
+    const Tile&,
+    Tile* const,
     FilterBuffer* input_metadata,
     FilterBuffer* input,
     FilterBuffer* output_metadata,
@@ -261,7 +265,7 @@ Status ChecksumSHA256Filter::compare_checksum_part(
     message << shastring_existing;
     message << " got ";
     message << shastring;
-    return Status::ChecksumError(message.str());
+    return Status_ChecksumError(message.str());
   }
 
   return Status::Ok();
