@@ -34,7 +34,7 @@
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/enums/datatype.h"
-#include "tiledb/sm/misc/utils.h"
+#include "tiledb/sm/misc/tdb_time.h"
 #include "tiledb/sm/misc/uuid.h"
 
 #include <iostream>
@@ -49,9 +49,17 @@ namespace sm {
 /*     CONSTRUCTORS & DESTRUCTORS    */
 /* ********************************* */
 
-Metadata::Metadata() {
-  auto t = utils::time::timestamp_now_ms();
-  timestamp_range_ = std::make_pair(t, t);
+Metadata::Metadata()
+    : Metadata(std::map<std::string, MetadataValue>()) {
+}
+
+Metadata::Metadata(const std::map<std::string, MetadataValue>& metadata_map)
+    : metadata_map_(metadata_map)
+    , timestamp_range_([]() -> std::pair<uint64_t, uint64_t> {
+      auto t = utils::time::timestamp_now_ms();
+      return std::make_pair(t, t);
+    }()) {
+  build_metadata_index();
 }
 
 Metadata::Metadata(const Metadata& rhs)
@@ -63,6 +71,15 @@ Metadata::Metadata(const Metadata& rhs)
     build_metadata_index();
 }
 
+Metadata& Metadata::operator=(const Metadata& other) {
+  metadata_map_ = other.metadata_map_;
+  timestamp_range_ = other.timestamp_range_;
+  loaded_metadata_uris_ = other.loaded_metadata_uris_;
+  uri_ = other.uri_;
+  build_metadata_index();
+  return *this;
+}
+
 Metadata::~Metadata() = default;
 
 /* ********************************* */
@@ -92,19 +109,19 @@ Status Metadata::generate_uri(const URI& array_uri) {
   std::stringstream ss;
   ss << "__" << timestamp_range_.first << "_" << timestamp_range_.second << "_"
      << uuid;
-  uri_ = array_uri.join_path(constants::array_metadata_folder_name)
+  uri_ = array_uri.join_path(constants::array_metadata_dir_name)
              .join_path(ss.str());
 
   return Status::Ok();
 }
 
-Status Metadata::deserialize(
-    const std::vector<tdb_shared_ptr<Buffer>>& metadata_buffs) {
-  clear();
-
+tuple<Status, optional<shared_ptr<Metadata>>> Metadata::deserialize(
+    const std::vector<shared_ptr<Buffer>>& metadata_buffs) {
+  std::map<std::string, MetadataValue> metadata_map;
   if (metadata_buffs.empty())
-    return Status::Ok();
+    return {Status::Ok(), make_shared<Metadata>(HERE())};
 
+  Status st;
   uint32_t key_len;
   char del;
   size_t value_len;
@@ -112,12 +129,12 @@ Status Metadata::deserialize(
     // Iterate over all items
     buff->reset_offset();
     while (buff->offset() != buff->size()) {
-      RETURN_NOT_OK(buff->read(&key_len, sizeof(uint32_t)));
+      RETURN_NOT_OK_TUPLE(buff->read(&key_len, sizeof(uint32_t)), nullopt);
       std::string key((const char*)buff->cur_data(), key_len);
       buff->advance_offset(key_len);
-      RETURN_NOT_OK(buff->read(&del, sizeof(char)));
+      RETURN_NOT_OK_TUPLE(buff->read(&del, sizeof(char)), nullopt);
 
-      metadata_map_.erase(key);
+      metadata_map.erase(key);
 
       // Handle deletion
       if (del)
@@ -125,24 +142,25 @@ Status Metadata::deserialize(
 
       MetadataValue value_struct;
       value_struct.del_ = del;
-      RETURN_NOT_OK(buff->read(&value_struct.type_, sizeof(char)));
-      RETURN_NOT_OK(buff->read(&value_struct.num_, sizeof(uint32_t)));
+      RETURN_NOT_OK_TUPLE(
+          buff->read(&value_struct.type_, sizeof(char)), nullopt);
+      RETURN_NOT_OK_TUPLE(
+          buff->read(&value_struct.num_, sizeof(uint32_t)), nullopt);
+
       if (value_struct.num_) {
         value_len = value_struct.num_ *
                     datatype_size(static_cast<Datatype>(value_struct.type_));
         value_struct.value_.resize(value_len);
-        RETURN_NOT_OK(buff->read((void*)value_struct.value_.data(), value_len));
+        RETURN_NOT_OK_TUPLE(
+            buff->read((void*)value_struct.value_.data(), value_len), nullopt);
       }
 
       // Insert to metadata
-      metadata_map_.emplace(std::make_pair(key, std::move(value_struct)));
+      metadata_map.emplace(std::make_pair(key, std::move(value_struct)));
     }
   }
 
-  build_metadata_index();
-  // Note: `metadata_map_` and `metadata_index_` are immutable after this point
-
-  return Status::Ok();
+  return {Status::Ok(), make_shared<Metadata>(HERE(), metadata_map)};
 }
 
 Status Metadata::serialize(Buffer* buff) const {
@@ -253,7 +271,7 @@ Status Metadata::get(
 
   if (index >= metadata_index_.size())
     return LOG_STATUS(
-        Status::MetadataError("Cannot get metadata; index out of bounds"));
+        Status_MetadataError("Cannot get metadata; index out of bounds"));
 
   // Get key
   auto& key_str = *(metadata_index_[index].first);
@@ -328,6 +346,12 @@ void Metadata::reset(uint64_t timestamp) {
   timestamp_range_ = std::make_pair(timestamp, timestamp);
 }
 
+void Metadata::reset(
+    const uint64_t timestamp_start, const uint64_t timestamp_end) {
+  clear();
+  timestamp_range_ = std::make_pair(timestamp_start, timestamp_end);
+}
+
 Metadata::iterator Metadata::begin() const {
   return metadata_map_.cbegin();
 }
