@@ -125,6 +125,12 @@ const std::string coords = "__coords";
 /** Special name reserved for the timestamp attribute. */
 const std::string timestamps = "__timestamps";
 
+/** Special name reserved for the delete timestamp attribute. */
+const std::string delete_timestamps = "__delete_timestamps";
+
+/** Special name reserved for the delete condition name attribute. */
+const std::string delete_condition_names = "__delete_condition_names";
+
 /** The size of a timestamp cell. */
 const uint64_t timestamp_size = sizeof(uint64_t);
 
@@ -212,6 +218,9 @@ const std::string ok_file_suffix = ".ok";
 /** Suffix for the special write files used in TileDB. */
 const std::string write_file_suffix = ".wrt";
 
+/** Suffix for the special delete files used in TileDB. */
+const std::string delete_file_suffix = ".del";
+
 /** Suffix for the special metadata files used in TileDB. */
 const std::string meta_file_suffix = ".meta";
 
@@ -364,6 +373,9 @@ const std::string filter_checksum_sha256_str = "CHECKSUM_SHA256";
 /** String describing FILTER_DICTIONARY. */
 const std::string filter_dictionary_str = "DICTIONARY_ENCODING";
 
+/** String describing FILTER_SCALE_FLOAT. */
+const std::string filter_scale_float_str = "SCALE_FLOAT";
+
 /** The string representation for FilterOption type compression_level. */
 const std::string filter_option_compression_level_str = "COMPRESSION_LEVEL";
 
@@ -371,11 +383,20 @@ const std::string filter_option_compression_level_str = "COMPRESSION_LEVEL";
 const std::string filter_option_bit_width_max_window_str =
     "BIT_WIDTH_MAX_WINDOW";
 
-/** The string representation for FilterOption type positive_delta_max_window.
+/** The string representation for FilterOption type positives_delta_max_window.
  */
 const std::string filter_option_positive_delta_max_window_str =
     "POSITIVE_DELTA_MAX_WINDOW";
 
+/** The string representation for FilterOption type scale_float_bytewidth. */
+const std::string filter_option_scale_float_bytewidth = "SCALE_FLOAT_BYTEWIDTH";
+
+/** The string representation for FilterOption type scale_float_factor. */
+const std::string filter_option_scale_float_factor = "SCALE_FLOAT_FACTOR";
+
+/** The string representation for FilterOption type scale_float_offset. */
+const std::string filter_option_scale_float_offset = "SCALE_FLOAT_OFFSET";
+
 /** The string representation for type int32. */
 const std::string int32_str = "INT32";
 
@@ -526,6 +547,15 @@ const std::string hilbert_str = "hilbert";
 /** The string representation of null. */
 const std::string null_str = "null";
 
+/** The string representation of unordered label. */
+const std::string label_unordered_str = "unordered";
+
+/** The string representation of increasing order label. */
+const std::string label_increasing_str = "increasing";
+
+/** The string representation of decreasing order label. */
+const std::string label_decreasing_str = "decreasing";
+
 /** The string representation for object type invalid. */
 const std::string object_type_invalid_str = "INVALID";
 
@@ -570,13 +600,13 @@ const int32_t library_version[3] = {
     TILEDB_VERSION_MAJOR, TILEDB_VERSION_MINOR, TILEDB_VERSION_PATCH};
 
 /** The TileDB serialization format version number. */
-const uint32_t format_version = 14;
+const uint32_t format_version = 15;
 
 /** The lowest version supported for back compat writes. */
 const uint32_t back_compat_writes_min_format_version = 7;
 
 /** The lowest version supported for consolidation with timestamps. */
-const uint32_t consolidation_with_timestamps_min_version = 14;
+const uint32_t consolidation_with_timestamps_min_version = 15;
 
 /** The maximum size of a tile chunk (unit of compression) in bytes. */
 const uint64_t max_tile_chunk_size = 64 * 1024;
