@@ -122,6 +122,12 @@ extern const std::string coords;
 /** Special name reserved for the timestamp attribute. */
 extern const std::string timestamps;
 
+/** Special name reserved for the delete timestamp attribute. */
+extern const std::string delete_timestamps;
+
+/** Special name reserved for the delete condition name attribute. */
+extern const std::string delete_condition_names;
+
 /** The size of a timestamp cell. */
 extern const uint64_t timestamp_size;
 
@@ -197,6 +203,9 @@ extern const std::string ok_file_suffix;
 /** Suffix for the special write files used in TileDB. */
 extern const std::string write_file_suffix;
 
+/** Suffix for the special delete files used in TileDB. */
+extern const std::string delete_file_suffix;
+
 /** Suffix for the special metadata files used in TileDB. */
 extern const std::string meta_file_suffix;
 
@@ -350,6 +359,9 @@ extern const std::string filter_checksum_sha256_str;
 /** String describing FILTER_DICTIONARY. */
 extern const std::string filter_dictionary_str;
 
+/** String describing FILTER_SCALE_FLOAT. */
+extern const std::string filter_scale_float_str;
+
 /** The string representation for FilterOption type compression_level. */
 extern const std::string filter_option_compression_level_str;
 
@@ -360,6 +372,15 @@ extern const std::string filter_option_bit_width_max_window_str;
  */
 extern const std::string filter_option_positive_delta_max_window_str;
 
+/** The string representation for FilterOption type scale_float_bytewidth. */
+extern const std::string filter_option_scale_float_bytewidth;
+
+/** The string representation for FilterOption type scale_float_factor. */
+extern const std::string filter_option_scale_float_factor;
+
+/** The string representation for FilterOption type scale_float_offset. */
+extern const std::string filter_option_scale_float_offset;
+
 /** The string representation for type int32. */
 extern const std::string int32_str;
 
@@ -510,6 +531,15 @@ extern const std::string hilbert_str;
 /** The string representation of null. */
 extern const std::string null_str;
 
+/** The string representation of unordered label. */
+extern const std::string label_unordered_str;
+
+/** The string representation of increasing order label. */
+extern const std::string label_increasing_str;
+
+/** The string representation of decreasing order label. */
+extern const std::string label_decreasing_str;
+
 /** The string representation for object type invalid. */
 extern const std::string object_type_invalid_str;
 
