@@ -40,14 +40,14 @@
 #include "tiledb/sm/enums/query_type.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/parse_argument.h"
-#include "tiledb/sm/query/dense_reader.h"
-#include "tiledb/sm/query/global_order_writer.h"
-#include "tiledb/sm/query/ordered_writer.h"
+#include "tiledb/sm/query/legacy/reader.h"
 #include "tiledb/sm/query/query_condition.h"
-#include "tiledb/sm/query/reader.h"
-#include "tiledb/sm/query/sparse_global_order_reader.h"
-#include "tiledb/sm/query/sparse_unordered_with_dups_reader.h"
-#include "tiledb/sm/query/unordered_writer.h"
+#include "tiledb/sm/query/readers/dense_reader.h"
+#include "tiledb/sm/query/readers/sparse_global_order_reader.h"
+#include "tiledb/sm/query/readers/sparse_unordered_with_dups_reader.h"
+#include "tiledb/sm/query/writers/global_order_writer.h"
+#include "tiledb/sm/query/writers/ordered_writer.h"
+#include "tiledb/sm/query/writers/unordered_writer.h"
 #include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/tile/writer_tile.h"
@@ -89,7 +89,7 @@ Query::Query(
   auto st = array->get_query_type(&type_);
   assert(st.ok());
 
-  if (type_ == QueryType::WRITE) {
+  if (type_ != QueryType::READ) {
     subarray_ = Subarray(array_, stats_, logger_);
   } else {
     subarray_ = Subarray(array_, Layout::ROW_MAJOR, stats_, logger_);
@@ -131,21 +131,30 @@ Query::~Query() {
 
 Status Query::add_range(
     unsigned dim_idx, const void* start, const void* end, const void* stride) {
-  if (dim_idx >= array_schema_->dim_num())
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return logger_->status(
+        Status_QueryError("Adding a range for an unsupported query type"));
+  }
+
+  if (dim_idx >= array_schema_->dim_num()) {
     return logger_->status(
         Status_QueryError("Cannot add range; Invalid dimension index"));
+  }
 
-  if (start == nullptr || end == nullptr)
+  if (start == nullptr || end == nullptr) {
     return logger_->status(
         Status_QueryError("Cannot add range; Invalid range"));
+  }
 
-  if (stride != nullptr)
+  if (stride != nullptr) {
     return logger_->status(Status_QueryError(
         "Cannot add range; Setting range stride is currently unsupported"));
+  }
 
-  if (array_schema_->domain().dimension_ptr(dim_idx)->var_size())
+  if (array_schema_->domain().dimension_ptr(dim_idx)->var_size()) {
     return logger_->status(
         Status_QueryError("Cannot add range; Range must be fixed-sized"));
+  }
 
   // Prepare a temp range
   std::vector<uint8_t> range;
@@ -190,6 +199,11 @@ Status Query::add_range_var(
     uint64_t start_size,
     const void* end,
     uint64_t end_size) {
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return logger_->status(
+        Status_QueryError("Adding a range for an unsupported query type"));
+  }
+
   if (dim_idx >= array_schema_->dim_num())
     return logger_->status(
         Status_QueryError("Cannot add range; Invalid dimension index"));
@@ -238,10 +252,16 @@ Status Query::get_range(
     const void** start,
     const void** end,
     const void** stride) const {
-  if (type_ == QueryType::WRITE && !array_schema_->dense())
+  if (type_ != QueryType::READ) {
+    return logger_->status(Status_QueryError(
+        "Getting a var range size only applicable to read queries"));
+  }
+
+  if (type_ == QueryType::WRITE && !array_schema_->dense()) {
     return logger_->status(
         Status_QueryError("Getting a range from a write query is not "
                           "applicable to sparse arrays"));
+  }
 
   *stride = nullptr;
   return subarray_.get_range(dim_idx, range_idx, start, end);
@@ -252,9 +272,10 @@ Status Query::get_range_var_size(
     uint64_t range_idx,
     uint64_t* start_size,
     uint64_t* end_size) const {
-  if (type_ == QueryType::WRITE)
+  if (type_ != QueryType::READ) {
     return logger_->status(Status_QueryError(
-        "Getting a var range size from a write query is not applicable"));
+        "Getting a var range size only applicable to read queries"));
+  }
 
   return subarray_.get_range_var_size(dim_idx, range_idx, start_size, end_size);
   ;
@@ -262,9 +283,10 @@ Status Query::get_range_var_size(
 
 Status Query::get_range_var(
     unsigned dim_idx, uint64_t range_idx, void* start, void* end) const {
-  if (type_ == QueryType::WRITE)
+  if (type_ != QueryType::READ) {
     return logger_->status(Status_QueryError(
-        "Getting a var range from a write query is not applicable"));
+        "Getting a var range only applicable to read queries"));
+  }
 
   uint64_t start_size = 0;
   uint64_t end_size = 0;
@@ -354,38 +376,44 @@ Status Query::get_range_var_from_name(
 }
 
 Status Query::get_est_result_size(const char* name, uint64_t* size) {
-  if (type_ == QueryType::WRITE)
+  if (type_ != QueryType::READ) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Operation currently "
-        "unsupported for write queries"));
+        "only unsupported for read queries"));
+  }
 
-  if (name == nullptr)
+  if (name == nullptr) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Name cannot be null"));
+  }
 
   if (name == constants::coords &&
-      !array_schema_->domain().all_dims_same_type())
+      !array_schema_->domain().all_dims_same_type()) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Not applicable to zipped "
         "coordinates in arrays with heterogeneous domain"));
+  }
 
-  if (name == constants::coords && !array_schema_->domain().all_dims_fixed())
+  if (name == constants::coords && !array_schema_->domain().all_dims_fixed()) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Not applicable to zipped "
         "coordinates in arrays with domains with variable-sized dimensions"));
+  }
 
-  if (array_schema_->is_nullable(name))
+  if (array_schema_->is_nullable(name)) {
     return logger_->status(Status_QueryError(
         std::string(
             "Cannot get estimated result size; Input attribute/dimension '") +
         name + "' is nullable"));
+  }
 
   if (array_->is_remote() && !subarray_.est_result_size_computed()) {
     auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
+    if (rest_client == nullptr) {
       return logger_->status(
           Status_QueryError("Error in query estimate result size; remote "
                             "array with no rest client."));
+    }
 
     RETURN_NOT_OK(
         rest_client->get_query_est_result_sizes(array_->array_uri(), this));
@@ -397,23 +425,26 @@ Status Query::get_est_result_size(const char* name, uint64_t* size) {
 
 Status Query::get_est_result_size(
     const char* name, uint64_t* size_off, uint64_t* size_val) {
-  if (type_ == QueryType::WRITE)
+  if (type_ != QueryType::READ) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Operation currently "
-        "unsupported for write queries"));
+        "only supported for read queries"));
+  }
 
-  if (array_schema_->is_nullable(name))
+  if (array_schema_->is_nullable(name)) {
     return logger_->status(Status_QueryError(
         std::string(
             "Cannot get estimated result size; Input attribute/dimension '") +
         name + "' is nullable"));
+  }
 
   if (array_->is_remote() && !subarray_.est_result_size_computed()) {
     auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
+    if (rest_client == nullptr) {
       return logger_->status(
           Status_QueryError("Error in query estimate result size; remote "
                             "array with no rest client."));
+    }
 
     RETURN_NOT_OK(
         rest_client->get_query_est_result_sizes(array_->array_uri(), this));
@@ -425,31 +456,36 @@ Status Query::get_est_result_size(
 
 Status Query::get_est_result_size_nullable(
     const char* name, uint64_t* size_val, uint64_t* size_validity) {
-  if (type_ == QueryType::WRITE)
+  if (type_ != QueryType::READ) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Operation currently "
-        "unsupported for write queries"));
+        "only supported for read queries"));
+  }
 
-  if (name == nullptr)
+  if (name == nullptr) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Name cannot be null"));
+  }
 
-  if (!array_schema_->attribute(name))
+  if (!array_schema_->attribute(name)) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Nullable API is only"
         "applicable to attributes"));
+  }
 
-  if (!array_schema_->is_nullable(name))
+  if (!array_schema_->is_nullable(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get estimated result size; Input attribute '") +
         name + "' is not nullable"));
+  }
 
   if (array_->is_remote() && !subarray_.est_result_size_computed()) {
     auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
+    if (rest_client == nullptr) {
       return logger_->status(
           Status_QueryError("Error in query estimate result size; remote "
                             "array with no rest client."));
+    }
 
     return logger_->status(
         Status_QueryError("Error in query estimate result size; unimplemented "
@@ -465,27 +501,31 @@ Status Query::get_est_result_size_nullable(
     uint64_t* size_off,
     uint64_t* size_val,
     uint64_t* size_validity) {
-  if (type_ == QueryType::WRITE)
+  if (type_ != QueryType::READ) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Operation currently "
-        "unsupported for write queries"));
+        "only supported for read queries"));
+  }
 
-  if (!array_schema_->attribute(name))
+  if (!array_schema_->attribute(name)) {
     return logger_->status(Status_QueryError(
         "Cannot get estimated result size; Nullable API is only"
         "applicable to attributes"));
+  }
 
-  if (!array_schema_->is_nullable(name))
+  if (!array_schema_->is_nullable(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get estimated result size; Input attribute '") +
         name + "' is not nullable"));
+  }
 
   if (array_->is_remote() && !subarray_.est_result_size_computed()) {
     auto rest_client = storage_manager_->rest_client();
-    if (rest_client == nullptr)
+    if (rest_client == nullptr) {
       return logger_->status(
           Status_QueryError("Error in query estimate result size; remote "
                             "array with no rest client."));
+    }
 
     return logger_->status(
         Status_QueryError("Error in query estimate result size; unimplemented "
@@ -514,9 +554,10 @@ Query::get_max_mem_size_map() {
 }
 
 Status Query::get_written_fragment_num(uint32_t* num) const {
-  if (type_ != QueryType::WRITE)
+  if (type_ != QueryType::WRITE) {
     return logger_->status(Status_QueryError(
         "Cannot get number of fragments; Applicable only to WRITE mode"));
+  }
 
   *num = (uint32_t)written_fragment_info_.size();
 
@@ -524,9 +565,10 @@ Status Query::get_written_fragment_num(uint32_t* num) const {
 }
 
 Status Query::get_written_fragment_uri(uint32_t idx, const char** uri) const {
-  if (type_ != QueryType::WRITE)
+  if (type_ != QueryType::WRITE) {
     return logger_->status(Status_QueryError(
         "Cannot get fragment URI; Applicable only to WRITE mode"));
+  }
 
   auto num = (uint32_t)written_fragment_info_.size();
   if (idx >= num)
@@ -540,9 +582,10 @@ Status Query::get_written_fragment_uri(uint32_t idx, const char** uri) const {
 
 Status Query::get_written_fragment_timestamp_range(
     uint32_t idx, uint64_t* t1, uint64_t* t2) const {
-  if (type_ != QueryType::WRITE)
+  if (type_ != QueryType::WRITE) {
     return logger_->status(Status_QueryError(
         "Cannot get fragment timestamp range; Applicable only to WRITE mode"));
+  }
 
   auto num = (uint32_t)written_fragment_info_.size();
   if (idx >= num)
@@ -578,25 +621,28 @@ std::vector<std::string> Query::buffer_names() const {
   }
 
   // Special zipped coordinates name
-  if (coords_info_.coords_buffer_)
+  if (coords_info_.coords_buffer_) {
     ret.push_back(constants::coords);
+  }
 
   return ret;
 }
 
 QueryBuffer Query::buffer(const std::string& name) const {
   // Special zipped coordinates
-  if (type_ == QueryType::WRITE && name == constants::coords)
+  if (type_ == QueryType::WRITE && name == constants::coords) {
     return QueryBuffer(
         coords_info_.coords_buffer_,
         nullptr,
         coords_info_.coords_buffer_size_,
         nullptr);
+  }
 
   // Attribute or dimension
   auto buf = buffers_.find(name);
-  if (buf != buffers_.end())
+  if (buf != buffers_.end()) {
     return buf->second;
+  }
 
   // Named buffer does not exist
   return QueryBuffer{};
@@ -643,19 +689,27 @@ Status Query::get_buffer(
     uint64_t** buffer_off_size,
     void** buffer_val,
     uint64_t** buffer_val_size) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
   // Check attribute
   if (name == constants::coords) {
     return logger_->status(
         Status_QueryError("Cannot get buffer; Coordinates are not var-sized"));
   }
   if (array_schema_->attribute(name) == nullptr &&
-      array_schema_->dimension_ptr(name) == nullptr)
+      array_schema_->dimension_ptr(name) == nullptr) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; Invalid attribute/dimension name '") +
         name + "'"));
-  if (!array_schema_->var_size(name))
+  }
+  if (!array_schema_->var_size(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; '") + name + "' is fixed-sized"));
+  }
 
   // Attribute or dimension
   auto it = buffers_.find(name);
@@ -678,19 +732,27 @@ Status Query::get_buffer(
 
 Status Query::get_offsets_buffer(
     const char* name, uint64_t** buffer_off, uint64_t** buffer_off_size) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
   // Check attribute
   if (name == constants::coords) {
     return logger_->status(
         Status_QueryError("Cannot get buffer; Coordinates are not var-sized"));
   }
   if (array_schema_->attribute(name) == nullptr &&
-      array_schema_->dimension_ptr(name) == nullptr)
+      array_schema_->dimension_ptr(name) == nullptr) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; Invalid attribute/dimension name '") +
         name + "'"));
-  if (!array_schema_->var_size(name))
+  }
+  if (!array_schema_->var_size(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; '") + name + "' is fixed-sized"));
+  }
 
   // Attribute or dimension
   auto it = buffers_.find(name);
@@ -709,6 +771,12 @@ Status Query::get_offsets_buffer(
 
 Status Query::get_data_buffer(
     const char* name, void** buffer, uint64_t** buffer_size) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
   // Check attribute
   if (name != constants::coords) {
     if (array_schema_->attribute(name) == nullptr &&
@@ -749,6 +817,12 @@ Status Query::get_validity_buffer(
     const char* name,
     uint8_t** buffer_validity_bytemap,
     uint64_t** buffer_validity_bytemap_size) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
   // Check attribute
   if (!array_schema_->is_nullable(name))
     return logger_->status(Status_QueryError(
@@ -807,17 +881,26 @@ Status Query::get_buffer(
     void** buffer,
     uint64_t** buffer_size,
     const ValidityVector** validity_vector) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
   // Check nullable attribute
-  if (array_schema_->attribute(name) == nullptr)
+  if (array_schema_->attribute(name) == nullptr) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; Invalid attribute name '") + name +
         "'"));
-  if (array_schema_->var_size(name))
+  }
+  if (array_schema_->var_size(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; '") + name + "' is var-sized"));
-  if (!array_schema_->is_nullable(name))
+  }
+  if (!array_schema_->is_nullable(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; '") + name + "' is non-nullable"));
+  }
 
   // Attribute or dimension
   auto it = buffers_.find(name);
@@ -843,17 +926,26 @@ Status Query::get_buffer(
     void** buffer_val,
     uint64_t** buffer_val_size,
     const ValidityVector** validity_vector) const {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot get buffer; Unsupported query type."));
+  }
+
   // Check attribute
-  if (array_schema_->attribute(name) == nullptr)
+  if (array_schema_->attribute(name) == nullptr) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; Invalid attribute name '") + name +
         "'"));
-  if (!array_schema_->var_size(name))
+  }
+  if (!array_schema_->var_size(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; '") + name + "' is fixed-sized"));
-  if (!array_schema_->is_nullable(name))
+  }
+  if (!array_schema_->is_nullable(name)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot get buffer; '") + name + "' is non-nullable"));
+  }
 
   // Attribute or dimension
   auto it = buffers_.find(name);
@@ -883,8 +975,9 @@ Status Query::get_attr_serialization_state(
 }
 
 bool Query::has_results() const {
-  if (status_ == QueryStatus::UNINITIALIZED || type_ == QueryType::WRITE)
+  if (status_ == QueryStatus::UNINITIALIZED || type_ != QueryType::READ) {
     return false;
+  }
 
   for (const auto& it : buffers_) {
     if (*(it.second.buffer_size_) != 0)
@@ -924,14 +1017,18 @@ Status Query::init() {
 }
 
 URI Query::first_fragment_uri() const {
-  if (type_ == QueryType::WRITE || fragment_metadata_.empty())
+  if (type_ != QueryType::READ || fragment_metadata_.empty()) {
     return URI();
+  }
+
   return fragment_metadata_.front()->fragment_uri();
 }
 
 URI Query::last_fragment_uri() const {
-  if (type_ == QueryType::WRITE || fragment_metadata_.empty())
+  if (type_ != QueryType::READ || fragment_metadata_.empty()) {
     return URI();
+  }
+
   return fragment_metadata_.back()->fragment_uri();
 }
 
@@ -940,8 +1037,10 @@ Layout Query::layout() const {
 }
 
 const QueryCondition* Query::condition() const {
-  if (type_ == QueryType::WRITE)
+  if (type_ == QueryType::WRITE) {
     return nullptr;
+  }
+
   return &condition_;
 }
 
@@ -1036,11 +1135,15 @@ Status Query::create_strategy() {
     } else {
       assert(false);
     }
-  } else {
+  } else if (type_ == QueryType::READ) {
     bool use_default = true;
-    if (use_refactored_sparse_unordered_with_dups_reader() &&
-        !array_schema_->dense() && layout_ == Layout::UNORDERED &&
-        array_schema_->allows_dups()) {
+    bool all_dense = true;
+    for (auto& frag_md : fragment_metadata_) {
+      all_dense &= frag_md->dense();
+    }
+
+    if (use_refactored_sparse_unordered_with_dups_reader(
+            layout_, *array_schema_)) {
       use_default = false;
 
       auto&& [st, non_overlapping_ranges]{Query::non_overlapping_ranges()};
@@ -1073,33 +1176,19 @@ Status Query::create_strategy() {
             condition_));
       }
     } else if (
-        use_refactored_sparse_global_order_reader() && condition_.empty() &&
+        use_refactored_sparse_global_order_reader(layout_, *array_schema_) &&
         !array_schema_->dense() &&
-        (layout_ == Layout::GLOBAL_ORDER ||
-         (layout_ == Layout::UNORDERED && subarray_.range_num() <= 1))) {
+        (layout_ == Layout::GLOBAL_ORDER || layout_ == Layout::UNORDERED)) {
       // Using the reader for unordered queries to do deduplication.
       use_default = false;
-      strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
-          SparseGlobalOrderReader,
-          stats_->create_child("Reader"),
-          logger_,
-          storage_manager_,
-          array_,
-          config_,
-          buffers_,
-          subarray_,
-          layout_,
-          condition_,
-          consolidation_with_timestamps_));
-    } else if (use_refactored_dense_reader() && array_schema_->dense()) {
-      bool all_dense = true;
-      for (auto& frag_md : fragment_metadata_)
-        all_dense &= frag_md->dense();
 
-      if (all_dense) {
-        use_default = false;
+      auto&& [st, non_overlapping_ranges]{Query::non_overlapping_ranges()};
+      RETURN_NOT_OK(st);
+
+      if (*non_overlapping_ranges || !subarray_.is_set() ||
+          subarray_.range_num() == 1) {
         strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
-            DenseReader,
+            SparseGlobalOrderReader<uint8_t>,
             stats_->create_child("Reader"),
             logger_,
             storage_manager_,
@@ -1108,8 +1197,35 @@ Status Query::create_strategy() {
             buffers_,
             subarray_,
             layout_,
-            condition_));
+            condition_,
+            consolidation_with_timestamps_));
+      } else {
+        strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
+            SparseGlobalOrderReader<uint64_t>,
+            stats_->create_child("Reader"),
+            logger_,
+            storage_manager_,
+            array_,
+            config_,
+            buffers_,
+            subarray_,
+            layout_,
+            condition_,
+            consolidation_with_timestamps_));
       }
+    } else if (use_refactored_dense_reader(*array_schema_, all_dense)) {
+      use_default = false;
+      strategy_ = tdb_unique_ptr<IQueryStrategy>(tdb_new(
+          DenseReader,
+          stats_->create_child("Reader"),
+          logger_,
+          storage_manager_,
+          array_,
+          config_,
+          buffers_,
+          subarray_,
+          layout_,
+          condition_));
     }
 
     if (use_default) {
@@ -1125,6 +1241,9 @@ Status Query::create_strategy() {
           layout_,
           condition_));
     }
+  } else {
+    return logger_->status(
+        Status_QueryError("Cannot create strategy; unsupported query type"));
   }
 
   if (strategy_ == nullptr)
@@ -1151,7 +1270,7 @@ Status Query::disable_checks_consolidation() {
         "Cannot disable checks for consolidation after initialization"));
   }
 
-  if (type_ == QueryType::READ) {
+  if (type_ != QueryType::WRITE) {
     return logger_->status(Status_QueryError(
         "Cannot disable checks for consolidation; Applicable only to writes"));
   }
@@ -1208,6 +1327,11 @@ Status Query::check_buffer_names() {
 }
 
 Status Query::check_set_fixed_buffer(const std::string& name) {
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot set buffer; Unsupported query type."));
+  }
+
   if (name == constants::coords &&
       !array_schema_->domain().all_dims_same_type())
     return logger_->status(Status_QueryError(
@@ -1340,26 +1464,30 @@ Status Query::set_data_buffer(
   RETURN_NOT_OK(check_set_fixed_buffer(name));
 
   // Check buffer
-  if (check_null_buffers && buffer == nullptr)
-    if (type_ != QueryType::WRITE || *buffer_size != 0)
+  if (check_null_buffers && buffer == nullptr) {
+    if (type_ != QueryType::WRITE || *buffer_size != 0) {
       return logger_->status(
           Status_QueryError("Cannot set buffer; " + name + " buffer is null"));
+    }
+  }
 
   // Check buffer size
-  if (check_null_buffers && buffer_size == nullptr)
+  if (check_null_buffers && buffer_size == nullptr) {
     return logger_->status(Status_QueryError(
         "Cannot set buffer; " + name + " buffer size is null"));
+  }
 
   // For easy reference
   const bool is_dim = array_schema_->is_dim(name);
   const bool is_attr = array_schema_->is_attr(name);
 
   // Check that attribute/dimension exists
-  if (name != constants::coords && name != constants::timestamps && !is_dim &&
-      !is_attr)
+  if (name != constants::coords && name != constants::timestamps &&
+      name != constants::delete_timestamps && !is_dim && !is_attr) {
     return logger_->status(Status_QueryError(
         std::string("Cannot set buffer; Invalid attribute/dimension '") + name +
         "'"));
+  }
 
   if (array_schema_->dense() && type_ == QueryType::WRITE && !is_attr) {
     return logger_->status(Status_QueryError(
@@ -1368,17 +1496,19 @@ Status Query::set_data_buffer(
 
   // Check if zipped coordinates coexist with separate coordinate buffers
   if ((is_dim && has_zipped_coords_buffer_) ||
-      (name == constants::coords && has_coords_buffer_))
+      (name == constants::coords && has_coords_buffer_)) {
     return logger_->status(Status_QueryError(
         std::string("Cannot set separate coordinate buffers and "
                     "a zipped coordinate buffer in the same query")));
+  }
 
   // Error if setting a new attribute/dimension after initialization
   const bool exists = buffers_.find(name) != buffers_.end();
-  if (status_ != QueryStatus::UNINITIALIZED && !exists)
+  if (status_ != QueryStatus::UNINITIALIZED && !exists) {
     return logger_->status(Status_QueryError(
         std::string("Cannot set buffer for new attribute/dimension '") + name +
         "' after initialization"));
+  }
 
   if (name == constants::coords) {
     has_zipped_coords_buffer_ = true;
@@ -1536,6 +1666,12 @@ Status Query::set_buffer(
     void* const buffer_val,
     uint64_t* const buffer_val_size,
     const bool check_null_buffers) {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot set buffer; Unsupported query type."));
+  }
+
   // Check buffer
   if (check_null_buffers && buffer_val == nullptr)
     if (type_ != QueryType::WRITE || *buffer_val_size != 0)
@@ -1716,6 +1852,12 @@ Status Query::set_buffer(
     uint64_t* const buffer_val_size,
     ValidityVector&& validity_vector,
     const bool check_null_buffers) {
+  // Check query type
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot set layout; Unsupported query type."));
+  }
+
   // Check buffer
   if (check_null_buffers && buffer_val == nullptr)
     if (type_ != QueryType::WRITE || *buffer_val_size != 0)
@@ -1784,10 +1926,11 @@ Status Query::set_buffer(
 Status Query::set_est_result_size(
     std::unordered_map<std::string, Subarray::ResultSize>& est_result_size,
     std::unordered_map<std::string, Subarray::MemorySize>& max_mem_size) {
-  if (type_ == QueryType::WRITE)
-    return logger_->status(Status_QueryError(
-        "Cannot set estimated result size; Operation currently "
-        "unsupported for write queries"));
+  if (type_ != QueryType::READ) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot set estimated result size; Unsupported query type."));
+  }
+
   return subarray_.set_est_result_size(est_result_size, max_mem_size);
 }
 
@@ -1798,13 +1941,20 @@ Status Query::set_layout_unsafe(Layout layout) {
 }
 
 Status Query::set_layout(Layout layout) {
-  if (type_ == QueryType::READ && status_ != QueryStatus::UNINITIALIZED)
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot set layout; Unsupported query type."));
+  }
+
+  if (type_ == QueryType::READ && status_ != QueryStatus::UNINITIALIZED) {
     return logger_->status(
         Status_QueryError("Cannot set layout after initialization"));
+  }
 
-  if (layout == Layout::HILBERT)
+  if (layout == Layout::HILBERT) {
     return logger_->status(Status_QueryError(
         "Cannot set layout; Hilbert order is not applicable to queries"));
+  }
 
   if (type_ == QueryType::WRITE && array_schema_->dense() &&
       layout == Layout::UNORDERED) {
@@ -1820,8 +1970,8 @@ Status Query::set_layout(Layout layout) {
 Status Query::set_condition(const QueryCondition& condition) {
   if (type_ == QueryType::WRITE)
     return logger_->status(Status_QueryError(
-        "Cannot set query condition; Operation only applicable "
-        "to read queries"));
+        "Cannot set query condition; Operation not applicable "
+        "to write queries"));
 
   condition_ = condition;
   return Status::Ok();
@@ -1832,6 +1982,11 @@ void Query::set_status(QueryStatus status) {
 }
 
 Status Query::set_subarray(const void* subarray) {
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot set subarray; Unsupported query type."));
+  }
+
   if (!array_schema_->domain().all_dims_same_type())
     return logger_->status(
         Status_QueryError("Cannot set subarray; Function not applicable to "
@@ -1946,6 +2101,11 @@ Status Query::set_subarray_unsafe(const NDRange& subarray) {
 }
 
 Status Query::check_buffers_correctness() {
+  if (type_ != QueryType::READ && type_ != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot check buffers; Unsupported query type."));
+  }
+
   // Iterate through each attribute
   for (auto& attr : buffer_names()) {
     if (array_schema_->var_size(attr)) {
@@ -2060,33 +2220,35 @@ shared_ptr<Buffer> Query::rest_scratch() const {
   return rest_scratch_;
 }
 
-bool Query::use_refactored_dense_reader() {
-  bool use_refactored_readers = false;
+bool Query::use_refactored_dense_reader(
+    const ArraySchema& array_schema, bool all_dense) {
+  bool use_refactored_reader = false;
   bool found = false;
   // First check for legacy option
   config_.get<bool>(
-      "sm.use_refactored_readers", &use_refactored_readers, &found);
+      "sm.use_refactored_readers", &use_refactored_reader, &found);
   // If the legacy/deprecated option is set use it over the new parameters
   // This facilitates backwards compatibility
   if (found) {
     logger_->warn(
         "sm.use_refactored_readers config option is deprecated.\nPlease use "
         "'sm.query.dense.reader' with value of 'refactored' or 'legacy'");
-    return use_refactored_readers;
+  } else {
+    const std::string& val = config_.get("sm.query.dense.reader", &found);
+    assert(found);
+    use_refactored_reader = val == "refactored";
   }
 
-  const std::string& val = config_.get("sm.query.dense.reader", &found);
-  assert(found);
-
-  return val == "refactored";
+  return use_refactored_reader && array_schema.dense() && all_dense;
 }
 
-bool Query::use_refactored_sparse_global_order_reader() {
-  bool use_refactored_readers = false;
+bool Query::use_refactored_sparse_global_order_reader(
+    Layout layout, const ArraySchema& array_schema) {
+  bool use_refactored_reader = false;
   bool found = false;
   // First check for legacy option
   config_.get<bool>(
-      "sm.use_refactored_readers", &use_refactored_readers, &found);
+      "sm.use_refactored_readers", &use_refactored_reader, &found);
   // If the legacy/deprecated option is set use it over the new parameters
   // This facilitates backwards compatibility
   if (found) {
@@ -2094,23 +2256,24 @@ bool Query::use_refactored_sparse_global_order_reader() {
         "sm.use_refactored_readers config option is deprecated.\nPlease use "
         "'sm.query.sparse_global_order.reader' with value of 'refactored' or "
         "'legacy'");
-    return use_refactored_readers;
+  } else {
+    const std::string& val =
+        config_.get("sm.query.sparse_global_order.reader", &found);
+    assert(found);
+    use_refactored_reader = val == "refactored";
   }
-
-  const std::string& val =
-      config_.get("sm.query.sparse_global_order.reader", &found);
-  assert(found);
-
-  return val == "refactored";
+  return use_refactored_reader && !array_schema.dense() &&
+         (layout == Layout::GLOBAL_ORDER || layout == Layout::UNORDERED);
 }
 
-bool Query::use_refactored_sparse_unordered_with_dups_reader() {
-  bool use_refactored_readers = false;
+bool Query::use_refactored_sparse_unordered_with_dups_reader(
+    Layout layout, const ArraySchema& array_schema) {
+  bool use_refactored_reader = false;
   bool found = false;
 
   // First check for legacy option
   config_.get<bool>(
-      "sm.use_refactored_readers", &use_refactored_readers, &found);
+      "sm.use_refactored_readers", &use_refactored_reader, &found);
   // If the legacy/deprecated option is set use it over the new parameters
   // This facilitates backwards compatibility
   if (found) {
@@ -2118,14 +2281,15 @@ bool Query::use_refactored_sparse_unordered_with_dups_reader() {
         "sm.use_refactored_readers config option is deprecated.\nPlease use "
         "'sm.query.sparse_unordered_with_dups.reader' with value of "
         "'refactored' or 'legacy'");
-    return use_refactored_readers;
+  } else {
+    const std::string& val =
+        config_.get("sm.query.sparse_unordered_with_dups.reader", &found);
+    assert(found);
+    use_refactored_reader = val == "refactored";
   }
 
-  const std::string& val =
-      config_.get("sm.query.sparse_unordered_with_dups.reader", &found);
-  assert(found);
-
-  return val == "refactored";
+  return use_refactored_reader && !array_schema.dense() &&
+         layout == Layout::UNORDERED && array_schema.allows_dups();
 }
 
 tuple<Status, optional<bool>> Query::non_overlapping_ranges() {
