@@ -41,8 +41,8 @@
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/enums/query_condition_op.h"
 #include "tiledb/sm/query/ast/query_ast.h"
-#include "tiledb/sm/query/result_cell_slab.h"
-#include "tiledb/sm/query/result_tile.h"
+#include "tiledb/sm/query/readers/result_cell_slab.h"
+#include "tiledb/sm/query/readers/result_tile.h"
 
 using namespace tiledb::common;
 
@@ -60,6 +60,9 @@ class QueryCondition {
   /** Default constructor. */
   QueryCondition();
 
+  /** Constructor from a tree. */
+  QueryCondition(tdb_unique_ptr<tiledb::sm::ASTNode>&& tree);
+
   /** Copy constructor. */
   QueryCondition(const QueryCondition& rhs);
 
@@ -187,6 +190,11 @@ class QueryCondition {
       std::vector<BitmapType>& result_bitmap,
       uint64_t* cell_count);
 
+  /**
+   * Reverse the query condition using De Morgan's law.
+   */
+  QueryCondition negated_condition();
+
   /**
    * Sets the AST. This is internal state to only be used in
    * the serialization path.
@@ -245,10 +253,11 @@ class QueryCondition {
    * @param var_size The attribute is var sized or not.
    * @param nullable The attribute is nullable or not.
    * @param fill_value The fill value for the cells.
+   * @param combination_op The combination op.
    * @param result_cell_bitmap The input cell bitmap.
    * @return The filtered cell slabs.
    */
-  template <typename T, QueryConditionOp Op, typename ConditionOp>
+  template <typename T, QueryConditionOp Op, typename CombinationOp>
   void apply_ast_node(
       const tdb_unique_ptr<ASTNode>& node,
       uint64_t stride,
@@ -256,7 +265,7 @@ class QueryCondition {
       const bool nullable,
       const ByteVecValue& fill_value,
       const std::vector<ResultCellSlab>& result_cell_slabs,
-      ConditionOp condition_op,
+      CombinationOp combination_op,
       std::vector<uint8_t>& result_cell_bitmap) const;
 
   /**
@@ -267,6 +276,7 @@ class QueryCondition {
    * @param var_size The attribute is var sized or not.
    * @param nullable The attribute is nullable or not.
    * @param fill_value The fill value for the cells.
+   * @param combination_op The combination op.
    * @param result_cell_bitmap The input cell bitmap.
    * @return Status, filtered cell slabs.
    */
@@ -288,6 +298,7 @@ class QueryCondition {
    * @param node The value node to apply.
    * @param array_schema The current array schema.
    * @param stride The stride between cells.
+   * @param combination_op The combination op.
    * @param result_cell_bitmap The input cell bitmap.
    * @return Status, filtered cell slabs.
    */
@@ -306,10 +317,11 @@ class QueryCondition {
    *
    * @param node The node to apply.
    * @param array_schema The array schema associated with `result_cell_slabs`.
+   * @param stride The stride between cells.
+   * @param combination_op The combination op.
    * @param result_cell_bitmap A bitmap representation of cell slabs to filter.
    * Mutated to remove cell slabs that do not meet the criteria in this query
    * condition.
-   * @param stride The stride between cells.
    * @return Filtered cell slabs.
    */
   template <typename CombinationOp = std::logical_and<uint8_t>>
@@ -331,6 +343,7 @@ class QueryCondition {
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
    * @param var_size The attribute is var sized or not.
+   * @param combination_op The combination op.
    * @param result_buffer The result buffer.
    */
   template <typename T, QueryConditionOp Op, typename CombinationOp>
@@ -341,6 +354,7 @@ class QueryCondition {
       const uint64_t src_cell,
       const uint64_t stride,
       const bool var_size,
+      const bool nullable,
       CombinationOp combination_op,
       span<uint8_t> result_buffer) const;
 
@@ -353,6 +367,7 @@ class QueryCondition {
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
    * @param var_size The attribute is var sized or not.
+   * @param combination_op The combination op.
    * @param result_buffer The result buffer.
    * @return Status.
    */
@@ -364,6 +379,7 @@ class QueryCondition {
       const uint64_t src_cell,
       const uint64_t stride,
       const bool var_size,
+      const bool nullable,
       CombinationOp combination_op,
       span<uint8_t> result_buffer) const;
 
@@ -377,6 +393,7 @@ class QueryCondition {
    * @param start The start cell.
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
+   * @param combination_op The combination op.
    * @param result_buffer The result buffer.
    * @return Status.
    */
@@ -400,6 +417,7 @@ class QueryCondition {
    * @param start The start cell.
    * @param src_cell The cell offset in the source tile.
    * @param stride The stride between cells.
+   * @param combination_op The combination op.
    * @param result_buffer The buffer to use for results.
    * @return Void.
    */
@@ -421,13 +439,38 @@ class QueryCondition {
    * @param node The node to apply.
    * @param result_tile The result tile to get the cells from.
    * @param var_size The attribute is var sized or not.
+   * @param combination_op The combination op.
    * @param result_bitmap The result bitmap.
    */
   template <
       typename T,
       QueryConditionOp Op,
       typename BitmapType,
-      typename CombinationOp>
+      typename CombinationOp,
+      typename nullable>
+  void apply_ast_node_sparse(
+      const tdb_unique_ptr<ASTNode>& node,
+      ResultTile& result_tile,
+      const bool var_size,
+      CombinationOp combination_op,
+      std::vector<BitmapType>& result_bitmap) const;
+
+  /**
+   * Applies a value node on a sparse result tile,
+   * templated on the nullable operator.
+   *
+   * @param node The node to apply.
+   * @param result_tile The result tile to get the cells from.
+   * @param var_size The attribute is var sized or not.
+   * @param combination_op The combination op.
+   * @param result_bitmap The result bitmap.
+   * @return Status.
+   */
+  template <
+      typename T,
+      typename BitmapType,
+      typename CombinationOp,
+      typename nullable>
   void apply_ast_node_sparse(
       const tdb_unique_ptr<ASTNode>& node,
       ResultTile& result_tile,
@@ -441,14 +484,15 @@ class QueryCondition {
    * @param node The node to apply.
    * @param result_tile The result tile to get the cells from.
    * @param var_size The attribute is var sized or not.
+   * @param combination_op The combination op.
    * @param result_bitmap The result bitmap.
-   * @return Status.
    */
   template <typename T, typename BitmapType, typename CombinationOp>
   void apply_ast_node_sparse(
       const tdb_unique_ptr<ASTNode>& node,
       ResultTile& result_tile,
       const bool var_size,
+      const bool nullable,
       CombinationOp combination_op,
       std::vector<BitmapType>& result_bitmap) const;
 
@@ -459,6 +503,7 @@ class QueryCondition {
    * @param node The node to apply.
    * @param array_schema The current array schema.
    * @param result_tile The result tile to get the cells from.
+   * @param combination_op The combination op.
    * @param result_bitmap The result bitmap.
    * @return Status.
    */
@@ -476,6 +521,7 @@ class QueryCondition {
    * @param node The node to apply.
    * @param array_schema The array schema.
    * @param result_tile The result tile to get the cells from.
+   * @param combination_op The combination op.
    * @param result_bitmap The bitmap to use for results.
    * @return Void.
    */
