@@ -38,6 +38,7 @@
 #if !defined(NOMINMAX)
 #define NOMINMAX  // curl may include windows headers
 #endif
+
 #include <curl/curl.h>
 #include <cstdlib>
 #include <functional>
@@ -45,6 +46,9 @@
 #include <mutex>
 #include <string>
 #include <unordered_map>
+
+#include "tiledb/common/dynamic_memory/dynamic_memory.h"
+#include "tiledb/common/logger_public.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/buffer/buffer_list.h"
 #include "tiledb/sm/config/config.h"
@@ -97,7 +101,7 @@ size_t write_header_callback(
 class Curl {
  public:
   /** Constructor. */
-  Curl();
+  explicit Curl(const std::shared_ptr<Logger>& logger);
 
   /** Destructor. */
   ~Curl() = default;
@@ -129,6 +133,26 @@ class Curl {
    */
   std::string url_escape(const std::string& url) const;
 
+  /**
+   * Wrapper for sending patch request to server and returning
+   * the unbuffered response body.
+   *
+   * @param stats The stats instance to record into
+   * @param url URL to post to
+   * @param serialization_type Serialization type to use
+   * @param data Encoded data buffer for posting
+   * @param returned_data Buffer to store response data
+   * @param res_ns_uri Array Namespace and URI
+   * @return Status
+   */
+  Status patch_data(
+      stats::Stats* stats,
+      const std::string& url,
+      SerializationType serialization_type,
+      const BufferList* data,
+      Buffer* returned_data,
+      const std::string& res_ns_uri);
+
   /**
    * Wrapper for posting data to server and returning
    * the unbuffered response body.
@@ -149,6 +173,43 @@ class Curl {
       Buffer* returned_data,
       const std::string& res_ns_uri);
 
+  /**
+   * Wrapper for sending put request to server and returning
+   * the unbuffered response body.
+   *
+   * @param stats The stats instance to record into
+   * @param url URL to post to
+   * @param serialization_type Serialization type to use
+   * @param data Encoded data buffer for posting
+   * @param returned_data Buffer to store response data
+   * @param res_ns_uri Array Namespace and URI
+   * @return Status
+   */
+  Status put_data(
+      stats::Stats* stats,
+      const std::string& url,
+      SerializationType serialization_type,
+      const BufferList* data,
+      Buffer* returned_data,
+      const std::string& res_ns_uri);
+
+  /**
+   * Wrapper for sending options request to server.
+   *
+   * @param stats The stats instance to record into
+   * @param url URL to post to
+   * @param serialization_type Serialization type to use
+   * @param returned_data Buffer to store response data
+   * @param res_ns_uri Array Namespace and URI
+   * @return Status
+   */
+  Status options(
+      stats::Stats* const stats,
+      const std::string& url,
+      SerializationType serialization_type,
+      Buffer* returned_data,
+      const std::string& res_ns_uri);
+
   /**
    * Callback defined by the caller of the 'post_data' variant for
    * receiving buffered response data.
@@ -186,6 +247,36 @@ class Curl {
       PostResponseCb&& write_cb,
       const std::string& res_ns_uri);
 
+  /**
+   * Common code shared between variants of 'patch_data'.
+   *
+   * @param serialization_type Serialization type to use
+   * @param data Encoded data buffer for posting
+   * @param headers Request headers that must be freed after the curl
+   *    request is complete. If this routine returns a non-OK status,
+   *    the value returned through this parameter should be ignored.
+   * @return Status
+   */
+  Status patch_data_common(
+      SerializationType serialization_type,
+      const BufferList* data,
+      struct curl_slist** headers);
+
+  /**
+   * Common code shared between variants of 'put_data'.
+   *
+   * @param serialization_type Serialization type to use
+   * @param data Encoded data buffer for posting
+   * @param headers Request headers that must be freed after the curl
+   *    request is complete. If this routine returns a non-OK status,
+   *    the value returned through this parameter should be ignored.
+   * @return Status
+   */
+  Status put_data_common(
+      SerializationType serialization_type,
+      const BufferList* data,
+      struct curl_slist** headers);
+
   /**
    * Common code shared between variants of 'post_data'.
    *
@@ -236,6 +327,13 @@ class Curl {
       Buffer* returned_data,
       const std::string& res_ns_uri);
 
+  /**
+   * Get HTTP status code of last request
+   *
+   * @return tuple of status and last_request
+   */
+  tuple<Status, optional<long>> last_http_status_code();
+
  private:
   /** TileDB config parameters. */
   const Config* config_;
@@ -264,6 +362,15 @@ class Curl {
   /** List of http status codes to retry. */
   std::vector<uint32_t> retry_http_codes_;
 
+  /** The class logger. */
+  std::shared_ptr<Logger> logger_;
+
+  /** UID of the logger instance */
+  inline static std::atomic<uint64_t> logger_id_ = 0;
+
+  /** Verbose logging in curl. */
+  bool verbose_;
+
   /**
    * Populates the curl slist with authorization (token or username+password),
    * and any extra headers.
@@ -332,6 +439,19 @@ class Curl {
       size_t (*write_cb)(void*, size_t, size_t, void*),
       void* write_arg) const;
 
+  /**
+   * Common code shared between variants of 'make_curl_options_request'.
+   *
+   * @param stats The stats instance to record into
+   * @param url URL to fetch
+   * @param curl_code Set to the return value of the curl call
+   * @return Status
+   */
+  Status make_curl_request_options_common(
+      stats::Stats* const stats,
+      const char* const url,
+      CURLcode* const curl_code) const;
+
   /**
    * Check the given curl code for errors, returning a TileDB error status if
    * so.
