@@ -60,11 +60,11 @@
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/hash.h"
 #include "tiledb/sm/misc/parse_argument.h"
-#include "tiledb/sm/query/dense_reader.h"
-#include "tiledb/sm/query/reader.h"
-#include "tiledb/sm/query/sparse_global_order_reader.h"
-#include "tiledb/sm/query/sparse_unordered_with_dups_reader.h"
-#include "tiledb/sm/query/writer_base.h"
+#include "tiledb/sm/query/readers/dense_reader.h"
+#include "tiledb/sm/query/legacy/reader.h"
+#include "tiledb/sm/query/readers/sparse_global_order_reader.h"
+#include "tiledb/sm/query/readers/sparse_unordered_with_dups_reader.h"
+#include "tiledb/sm/query/writers/writer_base.h"
 #include "tiledb/sm/serialization/config.h"
 #include "tiledb/sm/serialization/query.h"
 #include "tiledb/sm/subarray/subarray.h"
@@ -1163,15 +1163,23 @@ Status query_to_capnp(
   auto layout = query.layout();
   auto type = query.type();
   auto array = query.array();
+  auto query_type = query.type();
 
-  if (layout == Layout::GLOBAL_ORDER && query.type() == QueryType::WRITE)
+  if (query_type != QueryType::READ && query_type != QueryType::WRITE) {
+    return LOG_STATUS(
+        Status_SerializationError("Cannot serialize; Unsupported query type."));
+  }
+
+  if (layout == Layout::GLOBAL_ORDER && query_type == QueryType::WRITE) {
     return LOG_STATUS(
         Status_SerializationError("Cannot serialize; global order "
                                   "serialization not supported for writes."));
+  }
 
-  if (array == nullptr)
+  if (array == nullptr) {
     return LOG_STATUS(
         Status_SerializationError("Cannot serialize; array is null."));
+  }
 
   const auto& schema = query.array_schema();
 
@@ -1243,9 +1251,8 @@ Status query_to_capnp(
     bool all_dense = true;
     for (auto& frag_md : array->fragment_metadata())
       all_dense &= frag_md->dense();
-    if (query.use_refactored_sparse_unordered_with_dups_reader() &&
-        !schema.dense() && layout == Layout::UNORDERED &&
-        schema.allows_dups()) {
+    if (query.use_refactored_sparse_unordered_with_dups_reader(
+            layout, schema)) {
       auto builder = query_builder->initReaderIndex();
 
       auto&& [st, non_overlapping_ranges]{query.non_overlapping_ranges()};
@@ -1269,21 +1276,31 @@ Status query_to_capnp(
         query_builder->setVarOffsetsBitsize(reader->offsets_bitsize());
         RETURN_NOT_OK(index_reader_to_capnp(query, *reader, &builder));
       }
-    } else if (
-        query.use_refactored_sparse_global_order_reader() && !schema.dense() &&
-        (query.condition() == nullptr || query.condition()->empty()) &&
-        (layout == Layout::GLOBAL_ORDER || layout == Layout::UNORDERED) &&
-        query.subarray()->range_num() <= 1) {
+    } else if (query.use_refactored_sparse_global_order_reader(
+                   layout, schema)) {
       auto builder = query_builder->initReaderIndex();
-      auto reader = (SparseGlobalOrderReader*)query.strategy();
 
-      query_builder->setVarOffsetsMode(reader->offsets_mode());
-      query_builder->setVarOffsetsAddExtraElement(
-          reader->offsets_extra_element());
-      query_builder->setVarOffsetsBitsize(reader->offsets_bitsize());
-      RETURN_NOT_OK(index_reader_to_capnp(query, *reader, &builder));
-    } else if (
-        query.use_refactored_dense_reader() && all_dense && schema.dense()) {
+      auto&& [st, non_overlapping_ranges]{query.non_overlapping_ranges()};
+      RETURN_NOT_OK(st);
+
+      if (*non_overlapping_ranges) {
+        auto reader = (SparseGlobalOrderReader<uint8_t>*)query.strategy();
+
+        query_builder->setVarOffsetsMode(reader->offsets_mode());
+        query_builder->setVarOffsetsAddExtraElement(
+            reader->offsets_extra_element());
+        query_builder->setVarOffsetsBitsize(reader->offsets_bitsize());
+        RETURN_NOT_OK(index_reader_to_capnp(query, *reader, &builder));
+      } else {
+        auto reader = (SparseGlobalOrderReader<uint64_t>*)query.strategy();
+
+        query_builder->setVarOffsetsMode(reader->offsets_mode());
+        query_builder->setVarOffsetsAddExtraElement(
+            reader->offsets_extra_element());
+        query_builder->setVarOffsetsBitsize(reader->offsets_bitsize());
+        RETURN_NOT_OK(index_reader_to_capnp(query, *reader, &builder));
+      }
+    } else if (query.use_refactored_dense_reader(schema, all_dense)) {
       auto builder = query_builder->initDenseReader();
       auto reader = (DenseReader*)query.strategy();
 
@@ -1341,17 +1358,24 @@ Status query_from_capnp(
   auto array = query->array();
   const auto& schema = query->array_schema();
 
-  if (array == nullptr)
+  if (array == nullptr) {
     return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; array pointer is null."));
+  }
 
   // Deserialize query type (sanity check).
   QueryType query_type = QueryType::READ;
   RETURN_NOT_OK(query_type_enum(query_reader.getType().cStr(), &query_type));
-  if (query_type != type)
+  if (query_type != type) {
     return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; Query opened for " + query_type_str(type) +
         " but got serialized type for " + query_reader.getType().cStr()));
+  }
+
+  if (query_type != QueryType::READ && query_type != QueryType::WRITE) {
+    return LOG_STATUS(Status_SerializationError(
+        "Cannot deserialize; Unsupported query type."));
+  }
 
   // Deserialize layout.
   Layout layout = Layout::UNORDERED;
@@ -1372,9 +1396,10 @@ Status query_from_capnp(
   }
 
   // Deserialize and set attribute buffers.
-  if (!query_reader.hasAttributeBufferHeaders())
+  if (!query_reader.hasAttributeBufferHeaders()) {
     return LOG_STATUS(Status_SerializationError(
         "Cannot deserialize; no attribute buffer headers in message."));
+  }
 
   auto buffer_headers = query_reader.getAttributeBufferHeaders();
   auto attribute_buffer_start = static_cast<char*>(buffer_start);
@@ -1807,26 +1832,52 @@ Status query_from_capnp(
       query->clear_strategy();
       RETURN_NOT_OK(query->set_layout_unsafe(layout));
 
+      auto&& [st, non_overlapping_ranges]{query->non_overlapping_ranges()};
+      RETURN_NOT_OK(st);
+
       auto reader_reader = query_reader.getReaderIndex();
-      auto reader = (SparseGlobalOrderReader*)query->strategy();
 
-      if (query_reader.hasVarOffsetsMode()) {
+      if (*non_overlapping_ranges) {
+        auto reader = (SparseGlobalOrderReader<uint8_t>*)query->strategy();
+
+        if (query_reader.hasVarOffsetsMode()) {
+          RETURN_NOT_OK(
+              reader->set_offsets_mode(query_reader.getVarOffsetsMode()));
+        }
+
+        RETURN_NOT_OK(reader->set_offsets_extra_element(
+            query_reader.getVarOffsetsAddExtraElement()));
+
+        if (query_reader.getVarOffsetsBitsize() > 0) {
+          RETURN_NOT_OK(
+              reader->set_offsets_bitsize(query_reader.getVarOffsetsBitsize()));
+        }
+
+        RETURN_NOT_OK(reader->initialize_memory_budget());
+
         RETURN_NOT_OK(
-            reader->set_offsets_mode(query_reader.getVarOffsetsMode()));
-      }
+            index_reader_from_capnp(schema, reader_reader, query, reader));
+      } else {
+        auto reader = (SparseGlobalOrderReader<uint64_t>*)query->strategy();
 
-      RETURN_NOT_OK(reader->set_offsets_extra_element(
-          query_reader.getVarOffsetsAddExtraElement()));
+        if (query_reader.hasVarOffsetsMode()) {
+          RETURN_NOT_OK(
+              reader->set_offsets_mode(query_reader.getVarOffsetsMode()));
+        }
+
+        RETURN_NOT_OK(reader->set_offsets_extra_element(
+            query_reader.getVarOffsetsAddExtraElement()));
+
+        if (query_reader.getVarOffsetsBitsize() > 0) {
+          RETURN_NOT_OK(
+              reader->set_offsets_bitsize(query_reader.getVarOffsetsBitsize()));
+        }
+
+        RETURN_NOT_OK(reader->initialize_memory_budget());
 
-      if (query_reader.getVarOffsetsBitsize() > 0) {
         RETURN_NOT_OK(
-            reader->set_offsets_bitsize(query_reader.getVarOffsetsBitsize()));
+            index_reader_from_capnp(schema, reader_reader, query, reader));
       }
-
-      RETURN_NOT_OK(reader->initialize_memory_budget());
-
-      RETURN_NOT_OK(
-          index_reader_from_capnp(schema, reader_reader, query, reader));
     } else if (
         query_reader.hasReaderIndex() && !schema.dense() &&
         layout == Layout::UNORDERED && schema.allows_dups()) {
