@@ -174,6 +174,16 @@ class StorageManager {
       const EncryptionKey& encryption_key,
       Metadata* metadata);
 
+  /**
+   * Load data from persistent storage.
+   *
+   * @param uri The object URI.
+   * @param encryption_key The encryption key to use.
+   * @return Status, Buffer with the data.
+   */
+  tuple<Status, optional<Buffer>> load_data_from_generic_tile(
+      const URI& uri, const EncryptionKey& encryption_key);
+
   /**
    * Load a group detail from URI
    *
@@ -940,6 +950,21 @@ class StorageManager {
   Status store_metadata(
       const URI& uri, const EncryptionKey& encryption_key, Metadata* metadata);
 
+  /**
+   * Stores data into persistent storage.
+   *
+   * @param data Data to store.
+   * @param size Size of the data.
+   * @param uri The object URI.
+   * @param encryption_key The encryption key to use.
+   * @return Status
+   */
+  Status store_data_to_generic_tile(
+      void* data,
+      const size_t size,
+      const URI& uri,
+      const EncryptionKey& encryption_key);
+
   /** Closes a file, flushing its contents to persistent storage. */
   Status close_file(const URI& uri);
 
@@ -1175,9 +1200,11 @@ class StorageManager {
    * @return Status, vector from the fragment name to the offset in `f_buff`
    *     where the basic fragment metadata starts.
    */
-  tuple<Status, optional<std::vector<std::pair<std::string, uint64_t>>>>
-  load_consolidated_fragment_meta(
-      const URI& uri, const EncryptionKey& enc_key, Buffer* f_buff);
+  tuple<
+      Status,
+      optional<Buffer>,
+      optional<std::vector<std::pair<std::string, uint64_t>>>>
+  load_consolidated_fragment_meta(const URI& uri, const EncryptionKey& enc_key);
 
   /** Block until there are zero in-progress queries. */
   void wait_for_zero_in_progress();
